from __future__ import unicode_literals

from ..models import Key

from .literals import TEST_KEY_PRIVATE_DATA


class KeyAPIViewTestMixin(object):
    def _request_test_key_create_view(self):
        return self.post(
            viewname='rest_api:key-list', data={
                'key_data': TEST_KEY_PRIVATE_DATA
            }
        )

    def _request_test_key_delete_view(self):
        return self.delete(
            viewname='rest_api:key-detail', kwargs={
                'pk': self.test_key_private.pk
            }
        )

    def _request_test_key_detail_view(self):
        return self.get(
            viewname='rest_api:key-detail', kwargs={
                'pk': self.test_key_private.pk
            }
        )


class KeyTestMixin(object):
    def _create_test_key_private(self):
        self.test_key_private = Key.objects.create(
            key_data=TEST_KEY_PRIVATE_DATA
        )


class KeyViewTestMixin(object):
    def _request_test_key_download_view(self):
        return self.get(
            viewname='django_gpg:key_download', kwargs={
                'pk': self.test_key_private.pk
            }
        )

    def _request_test_key_upload_view(self):
        return self.post(
            viewname='django_gpg:key_upload', data={
                'key_data': TEST_KEY_PRIVATE_DATA
            }
        )



from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('document_states', '0005_auto_20170803_0638'),
    ]

    operations = [
        migrations.RenameField(
            model_name='workflowtransitiontriggerevent',
            old_name='stored_event_type',
            new_name='event_type',
        ),
    ]



from __future__ import absolute_import, unicode_literals

from django.test import TestCase, TransactionTestCase

from mayan.apps.acls.tests.mixins import ACLTestCaseMixin
from mayan.apps.converter.tests.mixins import LayerTestCaseMixin
from mayan.apps.permissions.tests.mixins import PermissionTestCaseMixin
from mayan.apps.smart_settings.tests.mixins import SmartSettingsTestCaseMixin


from mayan.apps.user_management.tests.mixins import UserTestMixin

from .mixins import (
    ClientMethodsTestCaseMixin, ConnectionsCheckTestCaseMixin,
    ContentTypeCheckTestCaseMixin, DownloadTestCaseMixin, ModelTestCaseMixin,
    OpenFileCheckTestCaseMixin, RandomPrimaryKeyModelMonkeyPatchMixin,
    SilenceLoggerTestCaseMixin, TempfileCheckTestCasekMixin,
    TestViewTestCaseMixin
)


class BaseTestCaseMixin(
    LayerTestCaseMixin, SilenceLoggerTestCaseMixin,
    ConnectionsCheckTestCaseMixin, DownloadTestCaseMixin,
    RandomPrimaryKeyModelMonkeyPatchMixin, ACLTestCaseMixin,
    ModelTestCaseMixin, OpenFileCheckTestCaseMixin, PermissionTestCaseMixin,
    SmartSettingsTestCaseMixin, TempfileCheckTestCasekMixin, UserTestMixin,
):
    """
    This is the most basic test case mixin class any test in the project
    should use.
    """


class BaseTestCase(BaseTestCaseMixin, TestCase):
    """
    All the project test mixin on top of Django test case class.
    """


class BaseTransactionTestCase(BaseTestCaseMixin, TransactionTestCase):
    """
    All the project test mixin on top of Django transaction test case class.
    """


class GenericViewTestCase(
    ClientMethodsTestCaseMixin, ContentTypeCheckTestCaseMixin,
    TestViewTestCaseMixin, BaseTestCase
):
    """
    A generic view test case built on top of the base test case providing
    a single, user customizable view to test object resolution and shorthand
    HTTP method functions.
    """



from __future__ import unicode_literals

from mayan.apps.documents.tests.base import GenericDocumentViewTestCase

from ..permissions import permission_template_sandbox


class DocumentTemplateSandboxViewTestMixin(object):
    def _request_document_template_sandbox_view(self):
        return self.get(
            viewname='templating:document_template_sandbox',
            kwargs={'pk': self.test_document.pk}
        )


class DocumentTemplateSandboxViewTestCase(
    DocumentTemplateSandboxViewTestMixin, GenericDocumentViewTestCase
):
    def test_document_template_sanbox_view_no_permissions(self):
        response = self._request_document_template_sandbox_view()
        self.assertEqual(response.status_code, 404)

    def test_document_template_sanbox_view_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_template_sandbox
        )

        response = self._request_document_template_sandbox_view()
        self.assertEqual(response.status_code, 200)



from __future__ import absolute_import, unicode_literals

from mayan.apps.appearance.classes import Icon

icon_document_send = Icon(
    driver_name='fontawesome', symbol='envelope'
)
icon_document_send_link = Icon(
    driver_name='fontawesome', symbol='envelope'
)
icon_document_multiple_send = Icon(
    driver_name='fontawesome', symbol='envelope'
)
icon_document_multiple_send_link = Icon(
    driver_name='fontawesome', symbol='envelope'
)
icon_mail_document_submit = Icon(
    driver_name='fontawesome', symbol='envelope'
)
icon_system_mailer_error_log = Icon(
    driver_name='fontawesome', symbol='envelope'
)
icon_user_mailer_create = Icon(
    driver_name='fontawesome-dual', primary_symbol='envelope',
    secondary_symbol='plus'
)

icon_user_mailer_delete = Icon(driver_name='fontawesome', symbol='times')
icon_user_mailer_edit = Icon(driver_name='fontawesome', symbol='pencil-alt')
icon_user_mailer_list = Icon(driver_name='fontawesome', symbol='envelope')
icon_user_mailer_setup = Icon(driver_name='fontawesome', symbol='envelope')
icon_user_mailer_test = Icon(driver_name='fontawesome', symbol='play')



from __future__ import unicode_literals

from django.apps import apps
from django.utils.translation import ugettext_lazy as _

from mayan.apps.navigation.classes import Link

from .icons import (
    icon_about, icon_book, icon_current_user_locale_profile_details,
    icon_current_user_locale_profile_edit, icon_documentation,
    icon_forum, icon_license, icon_setup, icon_source_code, icon_support,
    icon_tools
)
from .permissions_runtime import permission_error_log_view


def get_kwargs_factory(variable_name):
    def get_kwargs(context):
        ContentType = apps.get_model(
            app_label='contenttypes', model_name='ContentType'
        )

        content_type = ContentType.objects.get_for_model(
            model=context[variable_name]
        )
        return {
            'app_label': '"{}"'.format(content_type.app_label),
            'model': '"{}"'.format(content_type.model),
            'object_id': '{}.pk'.format(variable_name)
        }

    return get_kwargs


link_about = Link(
    icon_class=icon_about, text=_('About this'), view='common:about_view'
)
link_book = Link(
    icon_class=icon_book, tags='new_window', text=_('Get the book'),
    url='https://mayan-edms.com/book/'
)
link_current_user_locale_profile_details = Link(
    icon_class=icon_current_user_locale_profile_details,
    text=_('Locale profile'),
    view='common:current_user_locale_profile_details'
)
link_current_user_locale_profile_edit = Link(
    icon_class=icon_current_user_locale_profile_edit,
    text=_('Edit locale profile'),
    view='common:current_user_locale_profile_edit'
)
link_documentation = Link(
    icon_class=icon_documentation, tags='new_window',
    text=_('Documentation'), url='https://docs.mayan-edms.com'
)
link_object_error_list = Link(
    icon_class_path='mayan.apps.common.icons.icon_object_error_list',
    kwargs=get_kwargs_factory('resolved_object'),
    permissions=(permission_error_log_view,), text=_('Errors'),
    view='common:object_error_list',
)
link_object_error_list_clear = Link(
    icon_class_path='mayan.apps.common.icons.icon_object_error_list_clear',
    kwargs=get_kwargs_factory('resolved_object'),
    permissions=(permission_error_log_view,), text=_('Clear all'),
    view='common:object_error_list_clear',
)
link_forum = Link(
    icon_class=icon_forum, tags='new_window', text=_('Forum'),
    url='https://forum.mayan-edms.com'
)
link_license = Link(
    icon_class=icon_license, text=_('License'), view='common:license_view'
)
link_setup = Link(
    icon_class=icon_setup, text=_('Setup'), view='common:setup_list'
)
link_source_code = Link(
    icon_class=icon_source_code, tags='new_window', text=_('Source code'),
    url='https://gitlab.com/mayan-edms/mayan-edms'
)
link_support = Link(
    icon_class=icon_support, tags='new_window', text=_('Support'),
    url='http://www.mayan-edms.com/providers/'
)
link_tools = Link(
    icon_class=icon_tools, text=_('Tools'), view='common:tools_list'
)



from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('documents', '0049_auto_20190715_0454'),
    ]

    operations = [
        migrations.AlterField(
            model_name='document',
            name='language',
            field=models.CharField(
                blank=True, default='eng',
                help_text='The dominant language in the document.',
                max_length=8, verbose_name='Language'
            ),
        ),
    ]



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from mayan.apps.navigation.classes import Link

from .permissions import permission_template_sandbox


link_document_template_sandbox = Link(
    args='resolved_object.pk',
    icon_class_path='mayan.apps.templating.icons.icon_template_sandbox',
    permissions=(permission_template_sandbox,),
    text=_('Sandbox'),
    view='templating:document_template_sandbox',
)



from __future__ import absolute_import, unicode_literals

import logging

from django.contrib import messages
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.template import RequestContext
from django.urls import reverse
from django.utils.encoding import force_text
from django.utils.translation import ugettext_lazy as _

from mayan.apps.acls.models import AccessControlList
from mayan.apps.common.generics import (
    ConfirmView, FormView, SingleObjectCreateView, SingleObjectDeleteView,
    SingleObjectDetailView, SingleObjectDownloadView, SingleObjectListView
)
from mayan.apps.common.mixins import ExternalObjectMixin
from mayan.apps.django_gpg.exceptions import NeedPassphrase, PassphraseError
from mayan.apps.django_gpg.permissions import permission_key_sign
from mayan.apps.documents.models import DocumentVersion

from .forms import (
    DocumentVersionSignatureCreateForm,
    DocumentVersionSignatureDetailForm
)
from .icons import (
    icon_document_signature_list,
    icon_document_version_signature_detached_create,
    icon_document_version_signature_embedded_create
)
from .links import (
    link_document_version_signature_detached_create,
    link_document_version_signature_embedded_create,
    link_document_version_signature_upload
)
from .models import DetachedSignature, EmbeddedSignature, SignatureBaseModel
from .permissions import (
    permission_document_version_sign_detached,
    permission_document_version_sign_embedded,
    permission_document_version_signature_delete,
    permission_document_version_signature_download,
    permission_document_version_signature_upload,
    permission_document_version_signature_verify,
    permission_document_version_signature_view,
)
from .tasks import task_verify_missing_embedded_signature

logger = logging.getLogger(__name__)


class DocumentVersionDetachedSignatureCreateView(FormView):
    form_class = DocumentVersionSignatureCreateForm

    def form_valid(self, form):
        key = form.cleaned_data['key']
        passphrase = form.cleaned_data['passphrase'] or None

        AccessControlList.objects.check_access(
            obj=key, permissions=(permission_key_sign,), user=self.request.user
        )

        try:
            DetachedSignature.objects.sign_document_version(
                document_version=self.get_document_version(),
                key=key, passphrase=passphrase, user=self.request.user
            )
        except NeedPassphrase:
            messages.error(
                message=_('Passphrase is needed to unlock this key.'),
                request=self.request
            )
            return HttpResponseRedirect(
                redirect_to=reverse(
                    viewname='signatures:document_version_signature_detached_create',
                    kwargs={'pk': self.get_document_version().pk}
                )
            )
        except PassphraseError:
            messages.error(
                message=_('Passphrase is incorrect.'),
                request=self.request
            )
            return HttpResponseRedirect(
                redirect_to=reverse(
                    viewname='signatures:document_version_signature_detached_create',
                    kwargs={'pk': self.get_document_version().pk}
                )
            )
        else:
            messages.success(
                message=_('Document version signed successfully.'),
                request=self.request
            )

        return super(
            DocumentVersionDetachedSignatureCreateView, self
        ).form_valid(form)

    def dispatch(self, request, *args, **kwargs):
        AccessControlList.objects.check_access(
            obj=self.get_document_version().document,
            permissions=(permission_document_version_sign_detached,),
            user=request.user
        )

        return super(
            DocumentVersionDetachedSignatureCreateView, self
        ).dispatch(request, *args, **kwargs)

    def get_document_version(self):
        return get_object_or_404(klass=DocumentVersion, pk=self.kwargs['pk'])

    def get_extra_context(self):
        return {
            'object': self.get_document_version(),
            'submit_icon_class': icon_document_version_signature_detached_create,
            'submit_label': _('Sign'),
            'title': _(
                'Sign document version "%s" with a detached signature'
            ) % self.get_document_version(),
        }

    def get_form_extra_kwargs(self):
        return {'user': self.request.user}

    def get_post_action_redirect(self):
        return reverse(
            viewname='signatures:document_version_signature_list',
            kwargs={'pk': self.get_document_version().pk}
        )


class DocumentVersionEmbeddedSignatureCreateView(FormView):
    form_class = DocumentVersionSignatureCreateForm

    def form_valid(self, form):
        key = form.cleaned_data['key']
        passphrase = form.cleaned_data['passphrase'] or None

        AccessControlList.objects.check_access(
            obj=key, permissions=(permission_key_sign,), user=self.request.user
        )

        try:
            new_version = EmbeddedSignature.objects.sign_document_version(
                document_version=self.get_document_version(),
                key=key, passphrase=passphrase, user=self.request.user
            )
        except NeedPassphrase:
            messages.error(
                message=_('Passphrase is needed to unlock this key.'),
                request=self.request
            )
            return HttpResponseRedirect(
                redirect_to=reverse(
                    viewname='signatures:document_version_signature_embedded_create',
                    kwargs={'pk': self.get_document_version().pk}
                )
            )
        except PassphraseError:
            messages.error(
                message=_('Passphrase is incorrect.'),
                request=self.request
            )
            return HttpResponseRedirect(
                redirect_to=reverse(
                    viewname='signatures:document_version_signature_embedded_create',
                    kwargs={'pk': self.get_document_version().pk}
                )
            )
        else:
            messages.success(
                message=_('Document version signed successfully.'),
                request=self.request
            )

            return HttpResponseRedirect(
                redirect_to=reverse(
                    viewname='signatures:document_version_signature_list',
                    kwargs={'pk': new_version.pk}
                )
            )

        return super(
            DocumentVersionEmbeddedSignatureCreateView, self
        ).form_valid(form)

    def dispatch(self, request, *args, **kwargs):
        AccessControlList.objects.check_access(
            obj=self.get_document_version().document,
            permissions=(permission_document_version_sign_embedded,),
            user=request.user
        )

        return super(
            DocumentVersionEmbeddedSignatureCreateView, self
        ).dispatch(request, *args, **kwargs)

    def get_document_version(self):
        return get_object_or_404(klass=DocumentVersion, pk=self.kwargs['pk'])

    def get_extra_context(self):
        return {
            'object': self.get_document_version(),
            'submit_icon_class': icon_document_version_signature_embedded_create,
            'submit_label': _('Sign'),
            'title': _(
                'Sign document version "%s" with a embedded signature'
            ) % self.get_document_version(),
        }

    def get_form_extra_kwargs(self):
        return {'user': self.request.user}


class DocumentVersionSignatureDeleteView(SingleObjectDeleteView):
    model = DetachedSignature
    object_permission = permission_document_version_signature_delete

    def get_extra_context(self):
        return {
            'object': self.get_object().document_version,
            'signature': self.get_object(),
            'title': _('Delete detached signature: %s') % self.get_object()
        }

    def get_post_action_redirect(self):
        return reverse(
            viewname='signatures:document_version_signature_list',
            kwargs={'pk': self.get_object().document_version.pk}
        )


class DocumentVersionSignatureDetailView(SingleObjectDetailView):
    form_class = DocumentVersionSignatureDetailForm
    object_permission = permission_document_version_signature_view

    def get_extra_context(self):
        return {
            'hide_object': True,
            'object': self.get_object().document_version,
            'signature': self.get_object(),
            'title': _(
                'Details for signature: %s'
            ) % self.get_object(),
        }

    def get_source_queryset(self):
        return SignatureBaseModel.objects.select_subclasses()


class DocumentVersionSignatureDownloadView(SingleObjectDownloadView):
    model = DetachedSignature
    object_permission = permission_document_version_signature_download

    def get_download_file_object(self):
        return self.object.signature_file

    def get_download_filename(self):
        return force_text(self.object)


class DocumentVersionSignatureListView(
    ExternalObjectMixin, SingleObjectListView
):
    external_object_class = DocumentVersion
    external_object_permission = permission_document_version_signature_view

    def get_extra_context(self):
        return {
            'hide_object': True,
            'no_results_icon': icon_document_signature_list,
            'no_results_text': _(
                'Signatures help provide authorship evidence and tamper '
                'detection. They are very secure and hard to '
                'forge. A signature can be embedded as part of the document '
                'itself or uploaded as a separate file.'
            ),
            'no_results_secondary_links': [
                link_document_version_signature_detached_create.resolve(
                    RequestContext(
                        request=self.request, dict_={
                            'object': self.external_object
                        }
                    )
                ),
                link_document_version_signature_embedded_create.resolve(
                    RequestContext(
                        request=self.request, dict_={
                            'object': self.external_object
                        }
                    )
                ),
                link_document_version_signature_upload.resolve(
                    RequestContext(
                        request=self.request, dict_={
                            'object': self.external_object
                        }
                    )
                ),
            ],
            'no_results_title': _('There are no signatures for this document.'),
            'object': self.external_object,
            'title': _(
                'Signatures for document version: %s'
            ) % self.external_object,
        }

    def get_source_queryset(self):
        return self.external_object.signatures.all()


class DocumentVersionSignatureUploadView(SingleObjectCreateView):
    fields = ('signature_file',)
    model = DetachedSignature

    def dispatch(self, request, *args, **kwargs):
        AccessControlList.objects.check_access(
            obj=self.get_document_version(),
            permissions=(permission_document_version_signature_upload,),
            user=request.user
        )

        return super(
            DocumentVersionSignatureUploadView, self
        ).dispatch(request, *args, **kwargs)

    def get_document_version(self):
        return get_object_or_404(klass=DocumentVersion, pk=self.kwargs['pk'])

    def get_extra_context(self):
        return {
            'object': self.get_document_version(),
            'title': _(
                'Upload detached signature for document version: %s'
            ) % self.get_document_version(),
        }

    def get_instance_extra_data(self):
        return {'document_version': self.get_document_version()}

    def get_post_action_redirect(self):
        return reverse(
            viewname='signatures:document_version_signature_list',
            kwargs={'pk': self.get_document_version().pk}
        )


class AllDocumentSignatureVerifyView(ConfirmView):
    extra_context = {
        'message': _(
            'On large databases this operation may take some time to execute.'
        ), 'title': _('Verify all document for signatures?'),
    }
    view_permission = permission_document_version_signature_verify

    def get_post_action_redirect(self):
        return reverse(viewname='common:tools_list')

    def view_action(self):
        task_verify_missing_embedded_signature.delay()
        messages.success(
            message=_('Signature verification queued successfully.'),
            request=self.request
        )



from __future__ import unicode_literals

from django.dispatch import Signal

perform_upgrade = Signal(use_caching=True)
post_initial_setup = Signal(use_caching=True)
post_upgrade = Signal(use_caching=True)
pre_initial_setup = Signal(use_caching=True)
pre_upgrade = Signal(use_caching=True)



from __future__ import unicode_literals

import uuid

from pytz import common_timezones

from django.conf import settings
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.encoding import force_text, python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from .managers import ErrorLogEntryManager, UserLocaleProfileManager
from .storages import storage_sharedupload


def upload_to(instance, filename):
    return 'shared-file-{}'.format(uuid.uuid4().hex)


class ErrorLogEntry(models.Model):
    """
    Class to store an error log for any object. Uses generic foreign keys to
    reference the parent object.
    """
    namespace = models.CharField(
        max_length=128, verbose_name=_('Namespace')
    )
    content_type = models.ForeignKey(
        blank=True, on_delete=models.CASCADE, null=True,
        related_name='error_log_content_type', to=ContentType,
    )
    object_id = models.PositiveIntegerField(blank=True, null=True)
    content_object = GenericForeignKey(
        ct_field='content_type', fk_field='object_id',
    )
    datetime = models.DateTimeField(
        auto_now_add=True, db_index=True, verbose_name=_('Date time')
    )
    result = models.TextField(blank=True, null=True, verbose_name=_('Result'))

    objects = ErrorLogEntryManager()

    class Meta:
        ordering = ('datetime',)
        verbose_name = _('Error log entry')
        verbose_name_plural = _('Error log entries')


@python_2_unicode_compatible
class SharedUploadedFile(models.Model):
    """
    Keep a database link to a stored file. Used to share files between code
    that runs out of process.
    """
    file = models.FileField(
        storage=storage_sharedupload, upload_to=upload_to,
        verbose_name=_('File')
    )
    filename = models.CharField(max_length=255, verbose_name=_('Filename'))
    datetime = models.DateTimeField(
        auto_now_add=True, verbose_name=_('Date time')
    )

    class Meta:
        verbose_name = _('Shared uploaded file')
        verbose_name_plural = _('Shared uploaded files')

    def __str__(self):
        return self.filename

    def save(self, *args, **kwargs):
        self.filename = force_text(self.file)
        super(SharedUploadedFile, self).save(*args, **kwargs)

    def delete(self, *args, **kwargs):
        self.file.storage.delete(self.file.name)
        return super(SharedUploadedFile, self).delete(*args, **kwargs)

    def open(self):
        return self.file.storage.open(self.file.name)


@python_2_unicode_compatible
class UserLocaleProfile(models.Model):
    """
    Stores the locale preferences of a user. Stores timezone and language
    at the moment.
    """
    user = models.OneToOneField(
        on_delete=models.CASCADE, related_name='locale_profile',
        to=settings.AUTH_USER_MODEL, verbose_name=_('User')
    )
    timezone = models.CharField(
        choices=zip(common_timezones, common_timezones), max_length=48,
        verbose_name=_('Timezone')
    )
    language = models.CharField(
        choices=settings.LANGUAGES, max_length=8, verbose_name=_('Language')
    )

    objects = UserLocaleProfileManager()

    class Meta:
        verbose_name = _('User locale profile')
        verbose_name_plural = _('User locale profiles')

    def __str__(self):
        return force_text(self.user)

    def natural_key(self):
        return self.user.natural_key()
    natural_key.dependencies = [settings.AUTH_USER_MODEL]



from __future__ import unicode_literals

from django.contrib import admin

from .models import Cache


@admin.register(Cache)
class CacheAdmin(admin.ModelAdmin):
    list_display = ('name', 'label', 'storage_instance_path', 'maximum_size')



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from .classes import Layer
from .permissions import (
    permission_transformation_create, permission_transformation_delete,
    permission_transformation_edit, permission_transformation_view
)

layer_saved_transformations = Layer(
    default=True, label=_('Saved transformations'),
    name='saved_transformations', order=100, permissions={
        'create': permission_transformation_create,
        'delete': permission_transformation_delete,
        'edit': permission_transformation_edit,
        'select': permission_transformation_create,
        'view': permission_transformation_view,
    }, symbol='crop'
)



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from mayan.apps.common.apps import MayanAppConfig
from mayan.apps.common.html_widgets import TwoStateWidget
from mayan.apps.common.menus import menu_tools
from mayan.apps.navigation.classes import SourceColumn

from .classes import CeleryQueue, Task
from .links import link_task_manager
from .settings import *  # NOQA


class TaskManagerApp(MayanAppConfig):
    app_namespace = 'task_manager'
    app_url = 'task_manager'
    name = 'mayan.apps.task_manager'
    verbose_name = _('Task manager')

    def ready(self):
        super(TaskManagerApp, self).ready()

        CeleryQueue.initialize()

        SourceColumn(
            source=CeleryQueue, label=_('Label'), attribute='label'
        )
        SourceColumn(
            source=CeleryQueue, label=_('Name'), attribute='name'
        )
        SourceColumn(
            attribute='default_queue', label=_('Default queue?'),
            source=CeleryQueue, widget=TwoStateWidget
        )
        SourceColumn(
            attribute='transient', label=_('Is transient?'),
            source=CeleryQueue, widget=TwoStateWidget
        )
        SourceColumn(
            source=Task, label=_('Type'), attribute='task_type'
        )
        SourceColumn(
            source=Task, label=_('Start time'), attribute='get_time_started'
        )
        SourceColumn(
            source=Task, label=_('Host'),
            func=lambda context: context['object'].kwargs['hostname']
        )
        SourceColumn(
            source=Task, label=_('Arguments'),
            func=lambda context: context['object'].kwargs['args']
        )
        SourceColumn(
            source=Task, label=_('Keyword arguments'),
            func=lambda context: context['object'].kwargs['kwargs']
        )
        SourceColumn(
            source=Task, label=_('Worker process ID'),
            func=lambda context: context['object'].kwargs['worker_pid']
        )

        menu_tools.bind_links(links=(link_task_manager,))



from __future__ import unicode_literals

from django.conf.urls import url

from .views import (
    NamespaceDetailView, NamespaceListView, StatisticDetailView,
    StatisticQueueView
)

urlpatterns = [
    url(regex=r'^namespace/$', view=NamespaceListView.as_view(), name='namespace_list'),
    url(
        regex=r'^namespaces/(?P<slug>[\w-]+)/$',
        view=NamespaceDetailView.as_view(), name='namespace_details'
    ),
    url(
        regex=r'^statistics/(?P<slug>[\w-]+)/view/$', view=StatisticDetailView.as_view(),
        name='statistic_detail'
    ),
    url(
        regex=r'^statistics/(?P<slug>[\w-]+)/queue/$', view=StatisticQueueView.as_view(),
        name='statistic_queue'
    ),
]



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from mayan.apps.events.classes import EventTypeNamespace

namespace = EventTypeNamespace(
    label=_('Document parsing'), name='document_parsing'
)

event_parsing_document_content_deleted = namespace.add_event_type(
    label=_('Document parsed content deleted'),
    name='document_content_deleted'
)
event_parsing_document_version_submit = namespace.add_event_type(
    label=_('Document version submitted for parsing'), name='version_submit'
)
event_parsing_document_version_finish = namespace.add_event_type(
    label=_('Document version parsing finished'), name='version_finish'
)



from __future__ import unicode_literals

import json
import mock

from mayan.apps.common.tests.mixins import TestServerTestCaseMixin
from mayan.apps.common.tests.mocks import request_method_factory
from mayan.apps.document_states.tests.mixins import WorkflowTestMixin
from mayan.apps.documents.tests.base import GenericDocumentViewTestCase

from ..workflow_actions import HTTPPostAction

from .literals import (
    TEST_HEADERS_AUTHENTICATION_KEY, TEST_HEADERS_AUTHENTICATION_VALUE,
    TEST_HEADERS_KEY, TEST_HEADERS_JSON, TEST_HEADERS_JSON_TEMPLATE,
    TEST_HEADERS_JSON_TEMPLATE_KEY, TEST_HEADERS_VALUE, TEST_PAYLOAD_JSON,
    TEST_PAYLOAD_TEMPLATE_DOCUMENT_LABEL, TEST_SERVER_USERNAME,
    TEST_SERVER_PASSWORD
)


class HTTPPostWorkflowActionTestCase(
    TestServerTestCaseMixin, GenericDocumentViewTestCase, WorkflowTestMixin,
):
    auto_upload_document = False
    auto_add_test_view = True

    @mock.patch('requests.api.request')
    def test_http_post_action_simple(self, mock_object):
        mock_object.side_effect = request_method_factory(test_case=self)

        action = HTTPPostAction(
            form_data={
                'url': self.testserver_url,
            }
        )
        action.execute(context={})

        self.assertFalse(self.test_view_request is None)

    @mock.patch('requests.api.request')
    def test_http_post_action_payload_simple(self, mock_object):
        mock_object.side_effect = request_method_factory(test_case=self)

        action = HTTPPostAction(
            form_data={
                'url': self.testserver_url,
                'payload': TEST_PAYLOAD_JSON,
            }
        )
        action.execute(context={})

        self.assertEqual(
            json.loads(self.test_view_request.body),
            {'label': 'label'}
        )

    @mock.patch('requests.api.request')
    def test_http_post_action_payload_template(self, mock_object):
        self.upload_document()
        mock_object.side_effect = request_method_factory(test_case=self)

        action = HTTPPostAction(
            form_data={
                'url': self.testserver_url,
                'payload': TEST_PAYLOAD_TEMPLATE_DOCUMENT_LABEL,
            }
        )
        action.execute(context={'document': self.test_document})

        self.assertEqual(
            json.loads(self.test_view_request.body),
            {'label': self.test_document.label}
        )

    @mock.patch('requests.api.request')
    def test_http_post_action_headers_simple(self, mock_object):
        mock_object.side_effect = request_method_factory(test_case=self)

        action = HTTPPostAction(
            form_data={
                'url': self.testserver_url,
                'headers': TEST_HEADERS_JSON,
            }
        )
        action.execute(context={})

        self.assertTrue(
            TEST_HEADERS_KEY in self.test_view_request.META,
        )
        self.assertEqual(
            self.test_view_request.META[TEST_HEADERS_KEY], TEST_HEADERS_VALUE
        )

    @mock.patch('requests.api.request')
    def test_http_post_action_headers_template(self, mock_object):
        self.upload_document()
        mock_object.side_effect = request_method_factory(test_case=self)

        action = HTTPPostAction(
            form_data={
                'url': self.testserver_url,
                'headers': TEST_HEADERS_JSON_TEMPLATE,
            }
        )
        action.execute(context={'document': self.test_document})

        self.assertTrue(
            TEST_HEADERS_JSON_TEMPLATE_KEY in self.test_view_request.META,
        )
        self.assertEqual(
            self.test_view_request.META[TEST_HEADERS_JSON_TEMPLATE_KEY],
            self.test_document.label
        )

    @mock.patch('requests.api.request')
    def test_http_post_action_authentication(self, mock_object):
        mock_object.side_effect = request_method_factory(test_case=self)

        action = HTTPPostAction(
            form_data={
                'url': self.testserver_url,
                'username': TEST_SERVER_USERNAME,
                'password': TEST_SERVER_PASSWORD
            }
        )
        action.execute(context={})

        self.assertTrue(
            TEST_HEADERS_AUTHENTICATION_KEY in self.test_view_request.META,
        )
        self.assertEqual(
            self.test_view_request.META[TEST_HEADERS_AUTHENTICATION_KEY],
            TEST_HEADERS_AUTHENTICATION_VALUE
        )

    @mock.patch('requests.api.request')
    def test_http_post_action_timeout_value_int(self, mock_object):
        def mock_request(method, url, **kwargs):
            self.timeout = kwargs.get('timeout')

        mock_object.side_effect = mock_request

        action = HTTPPostAction(
            form_data={
                'url': self.testserver_url,
                'timeout': '1'
            }
        )
        action.execute(context={})

        self.assertEqual(self.timeout, 1)

    @mock.patch('requests.api.request')
    def test_http_post_action_timeout_value_float(self, mock_object):
        def mock_request(method, url, **kwargs):
            self.timeout = kwargs.get('timeout')

        mock_object.side_effect = mock_request

        action = HTTPPostAction(
            form_data={
                'url': self.testserver_url,
                'timeout': '1.5'
            }
        )
        action.execute(context={})

        self.assertEqual(self.timeout, 1.5)

    @mock.patch('requests.api.request')
    def test_http_post_action_timeout_value_none(self, mock_object):
        def mock_request(method, url, **kwargs):
            self.timeout = kwargs.get('timeout')

        mock_object.side_effect = mock_request

        action = HTTPPostAction(
            form_data={
                'url': self.testserver_url,
            }
        )
        action.execute(context={})

        self.assertEqual(self.timeout, None)



from __future__ import unicode_literals

import time

from unittest import skip
from django.test import override_settings
from django.utils.module_loading import import_string

from mayan.apps.common.tests.base import BaseTestCase

from ..exceptions import LockError

TEST_LOCK_1 = 'test lock 1'


class FileLockTestCase(BaseTestCase):
    backend_string = 'mayan.apps.lock_manager.backends.file_lock.FileLock'

    def setUp(self):
        super(FileLockTestCase, self).setUp()
        self.locking_backend = import_string(self.backend_string)

    def test_exclusive(self):
        lock_1 = self.locking_backend.acquire_lock(name=TEST_LOCK_1)
        with self.assertRaises(LockError):
            self.locking_backend.acquire_lock(name=TEST_LOCK_1)

        # Cleanup
        lock_1.release()

    def test_release(self):
        lock_1 = self.locking_backend.acquire_lock(name=TEST_LOCK_1)
        lock_1.release()
        lock_2 = self.locking_backend.acquire_lock(name=TEST_LOCK_1)

        # Cleanup
        lock_2.release()

    def test_timeout_expired(self):
        self.locking_backend.acquire_lock(name=TEST_LOCK_1, timeout=1)

        # lock_1 not release and not expired, should raise LockError
        with self.assertRaises(LockError):
            self.locking_backend.acquire_lock(name=TEST_LOCK_1)

        time.sleep(1.01)
        # lock_1 not release but has expired, should not raise LockError
        lock_2 = self.locking_backend.acquire_lock(name=TEST_LOCK_1)

        # Cleanup
        lock_2.release()

    def test_double_release(self):
        lock_1 = self.locking_backend.acquire_lock(name=TEST_LOCK_1)
        lock_1.release()

    def test_release_expired(self):
        lock_1 = self.locking_backend.acquire_lock(name=TEST_LOCK_1, timeout=1)
        time.sleep(1.01)
        lock_1.release()
        # No exception is raised even though the lock has expired.
        # The logic is that checking for expired locks during release is
        # not necessary as any attempt by someone else to aquire the lock
        # would be successfull, even after an extended lapse of time

    def test_release_expired_reaquired(self):
        self.locking_backend.acquire_lock(name=TEST_LOCK_1, timeout=1)
        time.sleep(1.01)
        # TEST_LOCK_1 is expired so trying to acquire it should not return an
        # error.
        lock_2 = self.locking_backend.acquire_lock(name=TEST_LOCK_1, timeout=1)

        # Cleanup
        lock_2.release()


class ModelLockTestCase(FileLockTestCase):
    backend_string = 'mayan.apps.lock_manager.backends.model_lock.ModelLock'


@skip('Skip until a Mock Redis server class is added.')
@override_settings(
    LOCK_MANAGER_BACKEND_ARGUMENTS={'redis_url': 'redis://127.0.0.1:6379/0'}
)
class RedisLockTestCase(FileLockTestCase):
    backend_string = 'mayan.apps.lock_manager.backends.redis_lock.RedisLock'



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from mayan.apps.documents.permissions import (
    permission_document_type_edit, permission_document_view
)
from mayan.apps.navigation.classes import Link


from .permissions import (
    permission_smart_link_create, permission_smart_link_delete,
    permission_smart_link_edit, permission_smart_link_view
)


link_document_type_smart_links = Link(
    args='resolved_object.pk',
    icon_class_path='mayan.apps.linking.icons.icon_document_type_smart_links',
    permissions=(permission_document_type_edit,), text=_('Smart links'),
    view='linking:document_type_smart_links',
)
link_smart_link_condition_create = Link(
    args='object.pk',
    icon_class_path='mayan.apps.linking.icons.icon_smart_link_condition_create',
    permissions=(permission_smart_link_edit,), text=_('Create condition'),
    view='linking:smart_link_condition_create',
)
link_smart_link_condition_delete = Link(
    args='resolved_object.pk',
    icon_class_path='mayan.apps.linking.icons.icon_smart_link_condition_delete',
    permissions=(permission_smart_link_edit,),
    tags='dangerous', text=_('Delete'),
    view='linking:smart_link_condition_delete',
)
link_smart_link_condition_edit = Link(
    args='resolved_object.pk',
    icon_class_path='mayan.apps.linking.icons.icon_smart_link_condition_edit',
    permissions=(permission_smart_link_edit,),
    text=_('Edit'), view='linking:smart_link_condition_edit',
)
link_smart_link_condition_list = Link(
    args='object.pk',
    icon_class_path='mayan.apps.linking.icons.icon_smart_link_condition_list',
    permissions=(permission_smart_link_edit,),
    text=_('Conditions'), view='linking:smart_link_condition_list',
)
link_smart_link_create = Link(
    icon_class_path='mayan.apps.linking.icons.icon_smart_link_create',
    permissions=(permission_smart_link_create,),
    text=_('Create new smart link'), view='linking:smart_link_create'
)
link_smart_link_delete = Link(
    args='object.pk',
    icon_class_path='mayan.apps.linking.icons.icon_smart_link_delete',
    permissions=(permission_smart_link_delete,),
    tags='dangerous', text=_('Delete'), view='linking:smart_link_delete',
)
link_smart_link_document_types = Link(
    args='object.pk',
    icon_class_path='mayan.apps.linking.icons.icon_smart_link_document_types',
    permissions=(permission_smart_link_edit,),
    text=_('Document types'), view='linking:smart_link_document_types',
)
link_smart_link_edit = Link(
    args='object.pk',
    icon_class_path='mayan.apps.linking.icons.icon_smart_link_edit',
    permissions=(permission_smart_link_edit,),
    text=_('Edit'), view='linking:smart_link_edit',
)
link_smart_link_instance_view = Link(
    args=('document.pk', 'object.pk',),
    permissions=(permission_smart_link_view,), text=_('Documents'),
    view='linking:smart_link_instance_view',
)
link_smart_link_instances_for_document = Link(
    args='resolved_object.pk',
    icon_class_path='mayan.apps.linking.icons.icon_smart_link_instances_for_document',
    permissions=(permission_document_view,), text=_('Smart links'),
    view='linking:smart_link_instances_for_document',
)
link_smart_link_list = Link(
    icon_class_path='mayan.apps.linking.icons.icon_smart_link_list',
    permissions=(permission_smart_link_create,), text=_('Smart links'),
    view='linking:smart_link_list'
)
link_smart_link_setup = Link(
    icon_class_path='mayan.apps.linking.icons.icon_smart_link_setup',
    permissions=(permission_smart_link_create,), text=_('Smart links'),
    view='linking:smart_link_list'
)



from __future__ import absolute_import, unicode_literals

import logging

from django.contrib import messages
from django.urls import reverse_lazy
from django.utils.translation import ugettext_lazy as _, ungettext

from mayan.apps.acls.models import AccessControlList
from mayan.apps.common.generics import (
    ConfirmView, MultipleObjectConfirmActionView
)
from mayan.apps.common.settings import setting_home_view

from ..icons import icon_document_list_deleted
from ..models import DeletedDocument, Document
from ..permissions import (
    permission_document_delete, permission_document_restore,
    permission_document_trash, permission_document_view,
    permission_empty_trash
)
from ..tasks import task_delete_document

from .document_views import DocumentListView

__all__ = (
    'DocumentTrashView', 'EmptyTrashCanView', 'TrashedDocumentDeleteView',
    'TrashedDocumentListView', 'TrashedDocumentRestoreView'
)
logger = logging.getLogger(__name__)


class DocumentTrashView(MultipleObjectConfirmActionView):
    model = Document
    object_permission = permission_document_trash
    pk_url_kwarg = 'pk'
    post_action_redirect = reverse_lazy(viewname=setting_home_view.value)
    success_message_singular = _(
        '%(count)d document moved to the trash.'
    )
    success_message_plural = _(
        '%(count)d documents moved to the trash.'
    )

    def get_extra_context(self):
        queryset = self.object_list

        result = {
            'title': ungettext(
                singular='Move the selected document to the trash?',
                plural='Move the selected documents to the trash?',
                number=queryset.count()
            )
        }

        return result

    def object_action(self, form, instance):
        instance.delete(_user=self.request.user)


class EmptyTrashCanView(ConfirmView):
    extra_context = {
        'title': _('Empty trash?')
    }
    view_permission = permission_empty_trash
    action_cancel_redirect = post_action_redirect = reverse_lazy(
        'documents:document_list_deleted'
    )

    def view_action(self):
        for deleted_document in DeletedDocument.objects.all():
            task_delete_document.apply_async(
                kwargs={'trashed_document_id': deleted_document.pk}
            )

        messages.success(
            message=_('Trash emptied successfully'), request=self.request
        )


class TrashedDocumentDeleteView(MultipleObjectConfirmActionView):
    model = DeletedDocument
    object_permission = permission_document_delete
    pk_url_kwarg = 'pk'
    success_message_singular = _(
        '%(count)d trashed document deleted.'
    )
    success_message_plural = _(
        '%(count)d trashed documents deleted.'
    )

    def get_extra_context(self):
        queryset = self.object_list

        result = {
            'title': ungettext(
                singular='Delete the selected trashed document?',
                plural='Delete the selected trashed documents?',
                number=queryset.count()
            )
        }

        return result

    def object_action(self, form, instance):
        task_delete_document.apply_async(
            kwargs={'trashed_document_id': instance.pk}
        )


class TrashedDocumentListView(DocumentListView):
    object_permission = None

    def get_document_queryset(self):
        return AccessControlList.objects.restrict_queryset(
            permission=permission_document_view,
            queryset=DeletedDocument.trash.all(),
            user=self.request.user
        )

    def get_extra_context(self):
        context = super(TrashedDocumentListView, self).get_extra_context()
        context.update(
            {
                'hide_link': True,
                'no_results_icon': icon_document_list_deleted,
                'no_results_text': _(
                    'To avoid loss of data, documents are not deleted '
                    'instantly. First, they are placed in the trash can. '
                    'From here they can be then finally deleted or restored.'
                ),
                'no_results_title': _(
                    'There are no documents in the trash can'
                ),
                'title': _('Documents in trash'),
            }
        )
        return context


class TrashedDocumentRestoreView(MultipleObjectConfirmActionView):
    model = DeletedDocument
    object_permission = permission_document_restore
    pk_url_kwarg = 'pk'
    success_message_singular = _(
        '%(count)d trashed document restored.'
    )
    success_message_plural = _(
        '%(count)d trashed documents restored.'
    )

    def get_extra_context(self):
        queryset = self.object_list

        result = {
            'title': ungettext(
                singular='Restore the selected trashed document?',
                plural='Restore the selected trashed documents?',
                number=queryset.count()
            )
        }

        return result

    def object_action(self, form, instance):
        instance.restore()



from __future__ import unicode_literals

import logging
import sys
import traceback

from django.apps import apps
from django.conf import settings
from django.db import models, transaction

from mayan.apps.documents.literals import DOCUMENT_IMAGE_TASK_TIMEOUT
from mayan.apps.documents.tasks import task_generate_document_page_image

from .events import (
    event_ocr_document_content_deleted, event_ocr_document_version_finish
)
from .runtime import ocr_backend
from .signals import post_document_version_ocr

logger = logging.getLogger(__name__)


class DocumentPageOCRContentManager(models.Manager):
    def delete_content_for(self, document, user=None):
        with transaction.atomic():
            for document_page in document.pages.all():
                self.filter(document_page=document_page).delete()

            event_ocr_document_content_deleted.commit(
                actor=user, target=document
            )

    def process_document_page(self, document_page):
        logger.info(
            'Processing page: %d of document version: %s',
            document_page.page_number, document_page.document_version
        )

        DocumentPageOCRContent = apps.get_model(
            app_label='ocr', model_name='DocumentPageOCRContent'
        )

        task = task_generate_document_page_image.apply_async(
            kwargs=dict(
                document_page_id=document_page.pk
            )
        )

        cache_filename = task.get(
            timeout=DOCUMENT_IMAGE_TASK_TIMEOUT, disable_sync_subtasks=False
        )

        with document_page.cache_partition.get_file(filename=cache_filename).open() as file_object:
            document_page_content, created = DocumentPageOCRContent.objects.get_or_create(
                document_page=document_page
            )
            document_page_content.content = ocr_backend.execute(
                file_object=file_object,
                language=document_page.document.language
            )
            document_page_content.save()

        logger.info(
            'Finished processing page: %d of document version: %s',
            document_page.page_number, document_page.document_version
        )

    def process_document_version(self, document_version):
        logger.info('Starting OCR for document version: %s', document_version)
        logger.debug('document version: %d', document_version.pk)

        try:
            with transaction.atomic():
                for document_page in document_version.pages.all():
                    self.process_document_page(document_page=document_page)

                logger.info(
                    'OCR complete for document version: %s', document_version
                )
                document_version.ocr_errors.all().delete()

                event_ocr_document_version_finish.commit(
                    action_object=document_version.document,
                    target=document_version
                )

                transaction.on_commit(
                    lambda: post_document_version_ocr.send(
                        sender=document_version.__class__,
                        instance=document_version
                    )
                )
        except Exception as exception:
            logger.error(
                'OCR error for document version: %d; %s', document_version.pk,
                exception
            )

            if settings.DEBUG:
                result = []
                type, value, tb = sys.exc_info()
                result.append('%s: %s' % (type.__name__, value))
                result.extend(traceback.format_tb(tb))
                document_version.ocr_errors.create(
                    result='\n'.join(result)
                )
            else:
                document_version.ocr_errors.create(result=exception)


class DocumentTypeSettingsManager(models.Manager):
    def get_by_natural_key(self, document_type_natural_key):
        DocumentType = apps.get_model(
            app_label='documents', model_name='DocumentType'
        )
        try:
            document_type = DocumentType.objects.get_by_natural_key(document_type_natural_key)
        except DocumentType.DoesNotExist:
            raise self.model.DoesNotExist

        return self.get(document_type__pk=document_type.pk)



from __future__ import unicode_literals

import pycountry

from django.apps import apps
from django.utils.translation import ugettext_lazy as _

from .literals import DOCUMENT_IMAGES_CACHE_NAME


def callback_update_cache_size(setting):
    Cache = apps.get_model(app_label='file_caching', model_name='Cache')
    cache = Cache.objects.get(name=DOCUMENT_IMAGES_CACHE_NAME)
    cache.maximum_size = setting.value
    cache.save()


def get_language(language_code):
    language = getattr(
        pycountry.languages.get(alpha_3=language_code), 'name', None
    )

    if language:
        return _(language)
    else:
        return _('Unknown language "%s"') % language_code


def get_language_choices():
    from .settings import setting_language_codes

    return sorted(
        [
            (
                iso639_3, _(pycountry.languages.get(alpha_3=iso639_3).name)
            ) for iso639_3 in setting_language_codes.value
        ], key=lambda x: x[1]
    )


def parse_range(astr):
    # http://stackoverflow.com/questions/4248399/
    # page-range-for-printing-algorithm
    result = set()
    for part in astr.split(','):
        x = part.split('-')
        result.update(range(int(x[0]), int(x[-1]) + 1))
    return sorted(result)



from __future__ import absolute_import, unicode_literals

import logging

from django.contrib import messages
from django.http import HttpResponseRedirect, JsonResponse
from django.shortcuts import get_object_or_404
from django.template import RequestContext
from django.urls import reverse, reverse_lazy
from django.utils.encoding import force_text
from django.utils.translation import ugettext_lazy as _

from mayan.apps.acls.models import AccessControlList
from mayan.apps.checkouts.models import NewVersionBlock
from mayan.apps.common.generics import (
    ConfirmView, MultiFormView, SingleObjectCreateView,
    SingleObjectDeleteView, SingleObjectEditView, SingleObjectListView
)
from mayan.apps.common.menus import menu_facet
from mayan.apps.common.mixins import ExternalObjectMixin
from mayan.apps.common.models import SharedUploadedFile
from mayan.apps.documents.models import DocumentType, Document
from mayan.apps.documents.permissions import (
    permission_document_create, permission_document_new_version
)
from mayan.apps.documents.tasks import task_upload_new_version
from mayan.apps.navigation.classes import Link

from .exceptions import SourceException
from .forms import (
    NewDocumentForm, NewVersionForm, WebFormUploadForm, WebFormUploadFormHTML5
)
from .icons import (
    icon_log, icon_setup_sources, icon_staging_folder_file,
    icon_upload_view_link
)
from .literals import SOURCE_UNCOMPRESS_CHOICE_ASK, SOURCE_UNCOMPRESS_CHOICE_Y
from .links import (
    link_setup_source_create_imap_email, link_setup_source_create_pop3_email,
    link_setup_source_create_staging_folder,
    link_setup_source_create_watch_folder, link_setup_source_create_webform,
    link_setup_source_create_sane_scanner
)
from .models import (
    InteractiveSource, Source, SaneScanner, StagingFolderSource
)
from .permissions import (
    permission_sources_setup_create, permission_sources_setup_delete,
    permission_sources_setup_edit, permission_sources_setup_view,
    permission_staging_file_delete
)
from .tasks import task_check_interval_source, task_source_handle_upload
from .utils import get_class, get_form_class, get_upload_form_class

logger = logging.getLogger(__name__)


class SourceLogListView(SingleObjectListView):
    view_permission = permission_sources_setup_view

    def get_extra_context(self):
        return {
            'hide_object': True,
            'no_results_icon': icon_log,
            'no_results_text': _(
                'Any error produced during the usage of a source will be '
                'listed here to assist in debugging.'
            ),
            'no_results_title': _('No log entries available'),
            'object': self.get_source(),
            'title': _('Log entries for source: %s') % self.get_source(),
        }

    def get_source_queryset(self):
        return self.get_source().logs.all()

    def get_source(self):
        return get_object_or_404(
            klass=Source.objects.select_subclasses(), pk=self.kwargs['pk']
        )


class UploadBaseView(MultiFormView):
    template_name = 'appearance/generic_form.html'
    prefixes = {'source_form': 'source', 'document_form': 'document'}

    @staticmethod
    def get_tab_link_for_source(source, document=None):
        if document:
            view = 'sources:document_version_upload'
            args = ('"{}"'.format(document.pk), '"{}"'.format(source.pk),)
        else:
            view = 'sources:document_upload_interactive'
            args = ('"{}"'.format(source.pk),)

        return Link(
            args=args,
            icon_class=icon_upload_view_link,
            keep_query=True,
            remove_from_query=['page'],
            text=source.label,
            view=view,
        )

    @staticmethod
    def get_active_tab_links(document=None):
        return [
            UploadBaseView.get_tab_link_for_source(source, document)
            for source in InteractiveSource.objects.filter(enabled=True).select_subclasses()
        ]

    def dispatch(self, request, *args, **kwargs):
        if 'source_id' in kwargs:
            self.source = get_object_or_404(
                klass=Source.objects.filter(enabled=True).select_subclasses(),
                pk=kwargs['source_id']
            )
        else:
            self.source = InteractiveSource.objects.filter(
                enabled=True
            ).select_subclasses().first()

        if not InteractiveSource.objects.filter(enabled=True).exists():
            messages.error(
                message=_(
                    'No interactive document sources have been defined or '
                    'none have been enabled, create one before proceeding.'
                ), request=request
            )
            return HttpResponseRedirect(
                redirect_to=reverse(viewname='sources:setup_source_list')
            )

        return super(UploadBaseView, self).dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(UploadBaseView, self).get_context_data(**kwargs)
        subtemplates_list = []

        context['source'] = self.source

        if isinstance(self.source, StagingFolderSource):
            try:
                staging_filelist = list(self.source.get_files())
            except Exception as exception:
                messages.error(message=exception, request=self.request)
                staging_filelist = []
            finally:
                subtemplates_list = [
                    {
                        'name': 'appearance/generic_multiform_subtemplate.html',
                        'context': {
                            'forms': context['forms'],
                            'title': _('Document properties'),
                        }
                    },
                    {
                        'name': 'appearance/generic_list_subtemplate.html',
                        'context': {
                            'hide_link': True,
                            'no_results_icon': icon_staging_folder_file,
                            'no_results_text': _(
                                'This could mean that the staging folder is '
                                'empty. It could also mean that the '
                                'operating system user account being used '
                                'for Mayan EDMS doesn\'t have the necessary '
                                'file system permissions for the folder.'
                            ),
                            'no_results_title': _(
                                'No staging files available'
                            ),
                            'object_list': staging_filelist,
                            'title': _('Files in staging path'),
                        }
                    },
                ]
        elif isinstance(self.source, SaneScanner):
            subtemplates_list.append({
                'name': 'sources/upload_multiform_subtemplate.html',
                'context': {
                    'forms': context['forms'],
                    'is_multipart': True,
                    'title': _('Document properties'),
                    'submit_label': _('Scan'),
                },
            })
        else:
            subtemplates_list.append({
                'name': 'sources/upload_multiform_subtemplate.html',
                'context': {
                    'forms': context['forms'],
                    'is_multipart': True,
                    'title': _('Document properties'),
                },
            })

        menu_facet.bound_links['sources:document_upload_interactive'] = self.tab_links
        menu_facet.bound_links['sources:document_version_upload'] = self.tab_links

        context.update(
            {
                'subtemplates_list': subtemplates_list,
            }
        )

        return context


class UploadInteractiveView(UploadBaseView):
    def dispatch(self, request, *args, **kwargs):
        self.subtemplates_list = []

        self.document_type = get_object_or_404(
            klass=DocumentType,
            pk=self.request.GET.get(
                'document_type_id', self.request.POST.get('document_type_id')
            )
        )

        AccessControlList.objects.check_access(
            obj=self.document_type, permissions=(permission_document_create,),
            user=request.user
        )

        self.tab_links = UploadBaseView.get_active_tab_links()

        try:
            return super(
                UploadInteractiveView, self
            ).dispatch(request, *args, **kwargs)
        except Exception as exception:
            if request.is_ajax():
                return JsonResponse(
                    data={'error': force_text(exception)}, status=500
                )
            else:
                raise

    def forms_valid(self, forms):
        if self.source.can_compress:
            if self.source.uncompress == SOURCE_UNCOMPRESS_CHOICE_ASK:
                expand = forms['source_form'].cleaned_data.get('expand')
            else:
                if self.source.uncompress == SOURCE_UNCOMPRESS_CHOICE_Y:
                    expand = True
                else:
                    expand = False
        else:
            expand = False

        try:
            uploaded_file = self.source.get_upload_file_object(
                forms['source_form'].cleaned_data
            )
        except SourceException as exception:
            messages.error(message=exception, request=self.request)
        else:
            shared_uploaded_file = SharedUploadedFile.objects.create(
                file=uploaded_file.file
            )

            if not self.request.user.is_anonymous:
                user_id = self.request.user.pk
            else:
                user_id = None

            try:
                self.source.clean_up_upload_file(uploaded_file)
            except Exception as exception:
                messages.error(message=exception, request=self.request)

            querystring = self.request.GET.copy()
            querystring.update(self.request.POST)

            try:
                task_source_handle_upload.apply_async(
                    kwargs=dict(
                        description=forms['document_form'].cleaned_data.get('description'),
                        document_type_id=self.document_type.pk,
                        expand=expand,
                        label=forms['document_form'].get_final_label(
                            filename=force_text(shared_uploaded_file)
                        ),
                        language=forms['document_form'].cleaned_data.get('language'),
                        querystring=querystring.urlencode(),
                        shared_uploaded_file_id=shared_uploaded_file.pk,
                        source_id=self.source.pk,
                        user_id=user_id,
                    )
                )
            except Exception as exception:
                message = _(
                    'Error executing document upload task; '
                    '%(exception)s, %(exception_class)s'
                ) % {
                    'exception': exception,
                    'exception_class': type(exception),
                }
                logger.critical(msg=message, exc_info=True)
                raise type(exception)(message)
            else:
                messages.success(
                    message=_(
                        'New document queued for upload and will be available '
                        'shortly.'
                    ), request=self.request
                )

        return HttpResponseRedirect(
            redirect_to='{}?{}'.format(
                reverse(
                    viewname=self.request.resolver_match.view_name,
                    kwargs=self.request.resolver_match.kwargs
                ), self.request.META['QUERY_STRING']
            ),
        )

    def create_source_form_form(self, **kwargs):
        if hasattr(self.source, 'uncompress'):
            show_expand = self.source.uncompress == SOURCE_UNCOMPRESS_CHOICE_ASK
        else:
            show_expand = False

        return self.get_form_classes()['source_form'](
            prefix=kwargs['prefix'],
            source=self.source,
            show_expand=show_expand,
            data=kwargs.get('data', None),
            files=kwargs.get('files', None),
        )

    def create_document_form_form(self, **kwargs):
        return self.get_form_classes()['document_form'](
            prefix=kwargs['prefix'],
            document_type=self.document_type,
            data=kwargs.get('data', None),
            files=kwargs.get('files', None),
        )

    def get_form_classes(self):
        source_form_class = get_upload_form_class(self.source.source_type)

        # Override source form class to enable the HTML5 file uploader
        if source_form_class == WebFormUploadForm:
            source_form_class = WebFormUploadFormHTML5

        return {
            'document_form': NewDocumentForm,
            'source_form': source_form_class
        }

    def get_context_data(self, **kwargs):
        context = super(UploadInteractiveView, self).get_context_data(**kwargs)
        context['title'] = _(
            'Upload a document of type "%(document_type)s" from '
            'source: %(source)s'
        ) % {'document_type': self.document_type, 'source': self.source.label}

        if not isinstance(self.source, StagingFolderSource) and not isinstance(self.source, SaneScanner):
            context['subtemplates_list'][0]['context'].update(
                {
                    'form_action': '{}?{}'.format(
                        reverse(
                            viewname=self.request.resolver_match.view_name,
                            kwargs=self.request.resolver_match.kwargs
                        ), self.request.META['QUERY_STRING']
                    ),
                    'form_css_classes': 'dropzone',
                    'form_disable_submit': True,
                    'form_id': 'html5upload',
                }
            )
        return context


class DocumentVersionUploadInteractiveView(UploadBaseView):
    def dispatch(self, request, *args, **kwargs):

        self.subtemplates_list = []

        self.document = get_object_or_404(
            klass=Document, pk=kwargs['document_pk']
        )

        if NewVersionBlock.objects.is_blocked(self.document):
            messages.error(
                message=_(
                    'Document "%s" is blocked from uploading new versions.'
                ) % self.document, request=self.request
            )
            return HttpResponseRedirect(
                redirect_to=reverse(
                    viewname='documents:document_version_list', kwargs={
                        'pk': self.document.pk
                    }
                )
            )

        AccessControlList.objects.check_access(
            obj=self.document, permissions=(permission_document_new_version,),
            user=self.request.user
        )

        self.tab_links = UploadBaseView.get_active_tab_links(self.document)

        return super(
            DocumentVersionUploadInteractiveView, self
        ).dispatch(request, *args, **kwargs)

    def forms_valid(self, forms):
        try:
            uploaded_file = self.source.get_upload_file_object(
                forms['source_form'].cleaned_data
            )
        except SourceException as exception:
            messages.error(message=exception, request=self.request)
        else:
            shared_uploaded_file = SharedUploadedFile.objects.create(
                file=uploaded_file.file
            )

            try:
                self.source.clean_up_upload_file(uploaded_file)
            except Exception as exception:
                messages.error(message=exception, request=self.request)

            if not self.request.user.is_anonymous:
                user_id = self.request.user.pk
            else:
                user_id = None

            task_upload_new_version.apply_async(kwargs=dict(
                shared_uploaded_file_id=shared_uploaded_file.pk,
                document_id=self.document.pk,
                user_id=user_id,
                comment=forms['document_form'].cleaned_data.get('comment')
            ))

            messages.success(
                message=_(
                    'New document version queued for upload and will be '
                    'available shortly.'
                ), request=self.request
            )

        return HttpResponseRedirect(
            redirect_to=reverse(
                viewname='documents:document_version_list', kwargs={
                    'pk': self.document.pk
                }
            )
        )

    def create_source_form_form(self, **kwargs):
        return self.get_form_classes()['source_form'](
            prefix=kwargs['prefix'],
            source=self.source,
            show_expand=False,
            data=kwargs.get('data', None),
            files=kwargs.get('files', None),
        )

    def get_form_classes(self):
        source_form_class = get_upload_form_class(self.source.source_type)

        # Override source form class to enable the HTML5 file uploader
        if source_form_class == WebFormUploadForm:
            source_form_class = WebFormUploadFormHTML5

        return {
            'document_form': NewVersionForm,
            'source_form': source_form_class
        }

    def get_context_data(self, **kwargs):
        context = super(
            DocumentVersionUploadInteractiveView, self
        ).get_context_data(**kwargs)
        context['object'] = self.document
        context['title'] = _(
            'Upload a new version for document "%(document)s" '
            'from source: %(source)s'
        ) % {'document': self.document, 'source': self.source.label}
        context['submit_label'] = _('Submit')
        context['form_css_classes'] = 'dropzone'
        context['form_disable_submit'] = True
        context['form_action'] = '{}?{}'.format(
            reverse(
                viewname=self.request.resolver_match.view_name,
                kwargs=self.request.resolver_match.kwargs
            ), self.request.META['QUERY_STRING']
        )

        return context


class StagingFileDeleteView(ExternalObjectMixin, SingleObjectDeleteView):
    external_object_class = StagingFolderSource
    external_object_permission = permission_staging_file_delete

    def get_extra_context(self):
        return {
            'object': self.object,
            'object_name': _('Staging file'),
            'title': _('Delete staging file "%s"?') % self.object,
        }

    def get_object(self):
        return self.external_object.get_file(
            encoded_filename=self.kwargs['encoded_filename']
        )


# Setup views
class SetupSourceCheckView(ConfirmView):
    """
    Trigger the task_check_interval_source task for a given source to
    test/debug their configuration irrespective of the schedule task setup.
    """
    view_permission = permission_sources_setup_create

    def get_extra_context(self):
        return {
            'object': self.get_object(),
            'subtitle': _(
                'This will execute the source check code even if the source '
                'is not enabled. Sources that delete content after '
                'downloading will not do so while being tested. Check the '
                'source\'s error log for information during testing. A '
                'successful test will clear the error log.'
            ), 'title': _(
                'Trigger check for source "%s"?'
            ) % self.get_object(),
        }

    def get_object(self):
        return get_object_or_404(
            klass=Source.objects.select_subclasses(), pk=self.kwargs['pk']
        )

    def view_action(self):
        task_check_interval_source.apply_async(
            kwargs={
                'source_id': self.get_object().pk, 'test': True
            }
        )

        messages.success(
            message=_('Source check queued.'), request=self.request
        )


class SetupSourceCreateView(SingleObjectCreateView):
    post_action_redirect = reverse_lazy(
        viewname='sources:setup_source_list'
    )
    view_permission = permission_sources_setup_create

    def get_form_class(self):
        return get_form_class(self.kwargs['source_type'])

    def get_extra_context(self):
        return {
            'object': self.kwargs['source_type'],
            'title': _(
                'Create new source of type: %s'
            ) % get_class(self.kwargs['source_type']).class_fullname(),
        }


class SetupSourceDeleteView(SingleObjectDeleteView):
    post_action_redirect = reverse_lazy(
        viewname='sources:setup_source_list'
    )
    view_permission = permission_sources_setup_delete

    def get_object(self):
        return get_object_or_404(
            klass=Source.objects.select_subclasses(), pk=self.kwargs['pk']
        )

    def get_form_class(self):
        return get_form_class(self.get_object().source_type)

    def get_extra_context(self):
        return {
            'object': self.get_object(),
            'title': _('Delete the source: %s?') % self.get_object(),
        }


class SetupSourceEditView(SingleObjectEditView):
    post_action_redirect = reverse_lazy(
        viewname='sources:setup_source_list'
    )
    view_permission = permission_sources_setup_edit

    def get_object(self):
        return get_object_or_404(
            klass=Source.objects.select_subclasses(), pk=self.kwargs['pk']
        )

    def get_form_class(self):
        return get_form_class(self.get_object().source_type)

    def get_extra_context(self):
        return {
            'object': self.get_object(),
            'title': _('Edit source: %s') % self.get_object(),
        }


class SetupSourceListView(SingleObjectListView):
    source_queryset = Source.objects.select_subclasses()
    view_permission = permission_sources_setup_view

    def get_extra_context(self):
        return {
            'hide_link': True,
            'hide_object': True,
            'no_results_icon': icon_setup_sources,
            'no_results_secondary_links': [
                link_setup_source_create_webform.resolve(
                    context=RequestContext(request=self.request)
                ),
                link_setup_source_create_imap_email.resolve(
                    context=RequestContext(request=self.request)
                ),
                link_setup_source_create_pop3_email.resolve(
                    context=RequestContext(request=self.request)
                ),
                link_setup_source_create_sane_scanner.resolve(
                    context=RequestContext(request=self.request)
                ),
                link_setup_source_create_staging_folder.resolve(
                    context=RequestContext(request=self.request)
                ),
                link_setup_source_create_watch_folder.resolve(
                    context=RequestContext(request=self.request)
                ),
            ],
            'no_results_text': _(
                'Sources provide the means to upload documents. '
                'Some sources like the webform, are interactive and require '
                'user input to operate. Others like the email sources, are '
                'automatic and run on the background without user intervention.'
            ),
            'no_results_title': _('No sources available'),
            'title': _('Sources'),
        }



from __future__ import unicode_literals

from django.conf.urls import url

from .api_views import (
    APICurrentUserView, APIGroupListView, APIGroupView, APIUserGroupList,
    APIUserListView, APIUserView
)
from .views import (
    CurrentUserDetailsView, CurrentUserEditView, GroupCreateView,
    GroupDeleteView, GroupEditView, GroupListView, GroupUsersView,
    UserCreateView, UserDeleteView, UserDetailsView, UserEditView,
    UserGroupsView, UserListView, UserOptionsEditView
)

urlpatterns_current_user = [
    url(
        regex=r'^user/$', view=CurrentUserDetailsView.as_view(),
        name='current_user_details'
    ),
    url(
        regex=r'^user/edit/$', view=CurrentUserEditView.as_view(),
        name='current_user_edit'
    )
]

urlpatterns_groups = [
    url(
        regex=r'^groups/$', view=GroupListView.as_view(), name='group_list'
    ),
    url(
        regex=r'^groups/create/$', view=GroupCreateView.as_view(),
        name='group_create'
    ),
    url(
        regex=r'^groups/(?P<pk>\d+)/delete/$', view=GroupDeleteView.as_view(),
        name='group_delete'
    ),
    url(
        regex=r'^groups/(?P<pk>\d+)/edit/$', view=GroupEditView.as_view(),
        name='group_edit'
    ),
    url(
        regex=r'^groups/(?P<pk>\d+)/users/$', view=GroupUsersView.as_view(),
        name='group_members'
    )
]

urlpatterns_users = [
    url(regex=r'^users/$', view=UserListView.as_view(), name='user_list'),
    url(
        regex=r'^users/create/$', view=UserCreateView.as_view(),
        name='user_create'
    ),
    url(
        regex=r'^users/(?P<pk>\d+)/delete/$', view=UserDeleteView.as_view(),
        name='user_delete'
    ),
    url(
        regex=r'^users/multiple/delete/$', view=UserDeleteView.as_view(),
        name='user_multiple_delete'
    ),
    url(
        regex=r'^users/(?P<pk>\d+)/$', view=UserDetailsView.as_view(),
        name='user_details'
    ),
    url(
        regex=r'^users/(?P<pk>\d+)/edit/$', view=UserEditView.as_view(),
        name='user_edit'
    ),
    url(
        regex=r'^users/(?P<pk>\d+)/groups/$', view=UserGroupsView.as_view(),
        name='user_groups'
    ),
    url(
        regex=r'^users/(?P<pk>\d+)/options/$',
        view=UserOptionsEditView.as_view(), name='user_options'
    ),
]

urlpatterns = []
urlpatterns.extend(urlpatterns_groups)
urlpatterns.extend(urlpatterns_current_user)
urlpatterns.extend(urlpatterns_users)

api_urls = [
    url(regex=r'^groups/$', view=APIGroupListView.as_view(), name='group-list'),
    url(
        regex=r'^groups/(?P<pk>[0-9]+)/$', view=APIGroupView.as_view(),
        name='group-detail'
    ),
    url(regex=r'^users/$', view=APIUserListView.as_view(), name='user-list'),
    url(
        regex=r'^users/(?P<pk>[0-9]+)/$', view=APIUserView.as_view(),
        name='user-detail'
    ),
    url(
        regex=r'^users/current/$', view=APICurrentUserView.as_view(),
        name='user-current'
    ),
    url(
        regex=r'^users/(?P<pk>[0-9]+)/groups/$',
        view=APIUserGroupList.as_view(), name='users-group-list'
    ),
]



from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('documents', '0032_auto_20160315_0537'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='documenttypefilename',
            options={
                'ordering': ('filename',), 'verbose_name': 'Quick label',
                'verbose_name_plural': 'Quick labels'
            },
        ),
    ]



from __future__ import unicode_literals

from django.db import migrations


def operation_move_from_content_type_user_to_foreign_key_field_user(apps, schema_editor):
    # The model references the use who checked out the document using a
    # generic.GenericForeignKey. This migrations changes that to a simpler
    # ForeignKey to the User model

    DocumentCheckout = apps.get_model(
        app_label='checkouts', model_name='DocumentCheckout'
    )

    for document_checkout in DocumentCheckout.objects.using(schema_editor.connection.alias).all():
        document_checkout.user = document_checkout.user_object
        document_checkout.save()


class Migration(migrations.Migration):

    dependencies = [
        ('checkouts', '0002_documentcheckout_user'),
    ]

    operations = [
        migrations.RunPython(
            code=operation_move_from_content_type_user_to_foreign_key_field_user
        ),
    ]



from __future__ import unicode_literals

from django import forms

from mayan.apps.common.widgets import TextAreaDiv

from .classes import Dependency


class DependenciesLicensesForm(forms.Form):
    text = forms.CharField(
        label='',
        widget=TextAreaDiv(
            attrs={
                'class': 'full-height scrollable',
                'data-height-difference': 270,
            }
        )
    )

    def __init__(self, *args, **kwargs):
        super(DependenciesLicensesForm, self).__init__(*args, **kwargs)
        copyright_texts = []

        for dependency in Dependency.get_all():
            copyright_text = dependency.get_copyright()
            if copyright_text:
                copyright_texts.append('-' * len(dependency.get_label()))
                copyright_texts.append(dependency.get_label().strip())
                copyright_texts.append('-' * len(dependency.get_label()))

                # Implement word wrapping at 79 columns.
                for line in copyright_text.split('\n'):
                    line_length = 0
                    new_line = []

                    for word in line.strip().split():
                        if line_length + len(word) > 79:
                            copyright_texts.append(' '.join(new_line))
                            new_line = [word]
                            line_length = 0
                        else:
                            new_line.append(word)
                            line_length = line_length + len(word)

                    copyright_texts.append(' '.join(new_line))

                copyright_texts.append('\n')

        self.fields['text'].initial = '\n'.join(copyright_texts)



from __future__ import unicode_literals

from django.conf.urls import url

from .views import (
    MayanLoginView, MayanLogoutView, MayanPasswordChangeDoneView,
    MayanPasswordChangeView, MayanPasswordResetCompleteView,
    MayanPasswordResetConfirmView, MayanPasswordResetDoneView,
    MayanPasswordResetView, UserSetPasswordView
)


urlpatterns = [
    url(regex=r'^login/$', view=MayanLoginView.as_view(), name='login_view'),
    url(
        regex=r'^logout/$', view=MayanLogoutView.as_view(), name='logout_view'
    ),
    url(
        regex=r'^password/change/done/$',
        view=MayanPasswordChangeDoneView.as_view(), name='password_change_done'
    ),
    url(
        regex=r'^password/change/$', view=MayanPasswordChangeView.as_view(),
        name='password_change_view'
    ),
    url(
        regex=r'^password/reset/complete/$',
        view=MayanPasswordResetCompleteView.as_view(),
        name='password_reset_complete_view'
    ),
    url(
        regex=r'^password/reset/confirm/(?P<uidb64>[0-9A-Za-z_\-]+)/(?P<token>[0-9A-Za-z]{1,13}-[0-9A-Za-z]{1,20})/$',
        view=MayanPasswordResetConfirmView.as_view(),
        name='password_reset_confirm_view'
    ),
    url(
        regex=r'^password/reset/done/$',
        view=MayanPasswordResetDoneView.as_view(),
        name='password_reset_done_view'
    ),
    url(
        regex=r'^password/reset/$', view=MayanPasswordResetView.as_view(),
        name='password_reset_view'
    ),
    url(
        regex=r'^users/(?P<pk>\d+)/set_password/$',
        view=UserSetPasswordView.as_view(), name='user_set_password'
    ),
    url(
        regex=r'^users/multiple/set_password/$',
        view=UserSetPasswordView.as_view(), name='user_multiple_set_password'
    ),
]



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from rest_framework import serializers
from rest_framework.exceptions import ValidationError
from rest_framework.reverse import reverse

from mayan.apps.documents.models import DocumentType
from mayan.apps.documents.serializers import (
    DocumentSerializer, DocumentTypeSerializer
)

from .models import SmartLink, SmartLinkCondition


class SmartLinkConditionSerializer(serializers.HyperlinkedModelSerializer):
    smart_link_url = serializers.SerializerMethodField()
    url = serializers.SerializerMethodField()

    class Meta:
        fields = (
            'enabled', 'expression', 'foreign_document_data', 'inclusion',
            'id', 'negated', 'operator', 'smart_link_url', 'url'
        )
        model = SmartLinkCondition

    def create(self, validated_data):
        validated_data['smart_link'] = self.context['smart_link']
        return super(SmartLinkConditionSerializer, self).create(validated_data)

    def get_smart_link_url(self, instance):
        return reverse(
            viewname='rest_api:smartlink-detail', args=(instance.smart_link.pk,),
            request=self.context['request'], format=self.context['format']
        )

    def get_url(self, instance):
        return reverse(
            viewname='rest_api:smartlinkcondition-detail', args=(
                instance.smart_link.pk, instance.pk,
            ), request=self.context['request'], format=self.context['format']
        )


class SmartLinkSerializer(serializers.HyperlinkedModelSerializer):
    conditions_url = serializers.HyperlinkedIdentityField(
        view_name='rest_api:smartlinkcondition-list'
    )
    document_types = DocumentTypeSerializer(read_only=True, many=True)

    class Meta:
        extra_kwargs = {
            'url': {'view_name': 'rest_api:smartlink-detail'},
        }
        fields = (
            'conditions_url', 'document_types', 'dynamic_label', 'enabled',
            'label', 'id', 'url'
        )
        model = SmartLink


class ResolvedSmartLinkDocumentSerializer(DocumentSerializer):
    resolved_smart_link_url = serializers.SerializerMethodField()

    class Meta(DocumentSerializer.Meta):
        fields = DocumentSerializer.Meta.fields + (
            'resolved_smart_link_url',
        )
        read_only_fields = DocumentSerializer.Meta.fields

    def get_resolved_smart_link_url(self, instance):
        return reverse(
            viewname='rest_api:resolvedsmartlink-detail', args=(
                self.context['document'].pk, self.context['smart_link'].pk
            ), request=self.context['request'],
            format=self.context['format']
        )


class ResolvedSmartLinkSerializer(SmartLinkSerializer):
    resolved_dynamic_label = serializers.SerializerMethodField()
    resolved_smart_link_url = serializers.SerializerMethodField()
    resolved_documents_url = serializers.SerializerMethodField()

    class Meta(SmartLinkSerializer.Meta):
        fields = SmartLinkSerializer.Meta.fields + (
            'resolved_dynamic_label', 'resolved_smart_link_url',
            'resolved_documents_url'
        )
        read_only_fields = SmartLinkSerializer.Meta.fields

    def get_resolved_documents_url(self, instance):
        return reverse(
            viewname='rest_api:resolvedsmartlinkdocument-list',
            args=(self.context['document'].pk, instance.pk,),
            request=self.context['request'], format=self.context['format']
        )

    def get_resolved_dynamic_label(self, instance):
        return instance.get_dynamic_label(document=self.context['document'])

    def get_resolved_smart_link_url(self, instance):
        return reverse(
            viewname='rest_api:resolvedsmartlink-detail',
            args=(self.context['document'].pk, instance.pk,),
            request=self.context['request'], format=self.context['format']
        )


class WritableSmartLinkSerializer(serializers.ModelSerializer):
    conditions_url = serializers.HyperlinkedIdentityField(
        view_name='rest_api:smartlinkcondition-list'
    )
    document_types_pk_list = serializers.CharField(
        help_text=_(
            'Comma separated list of document type primary keys to which this '
            'smart link will be attached.'
        ), required=False
    )

    class Meta:
        extra_kwargs = {
            'url': {'view_name': 'rest_api:smartlink-detail'},
        }
        fields = (
            'conditions_url', 'document_types_pk_list', 'dynamic_label',
            'enabled', 'label', 'id', 'url'
        )
        model = SmartLink

    def validate(self, attrs):
        document_types_pk_list = attrs.pop('document_types_pk_list', None)
        document_types_result = []

        if document_types_pk_list:
            for pk in document_types_pk_list.split(','):
                try:
                    document_type = DocumentType.objects.get(pk=pk)
                except DocumentType.DoesNotExist:
                    raise ValidationError(_('No such document type: %s') % pk)
                else:
                    # Accumulate valid stored document_type pks
                    document_types_result.append(document_type.pk)

        attrs['document_types'] = DocumentType.objects.filter(
            pk__in=document_types_result
        )
        return attrs



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from mayan.apps.dependencies.classes import (
    GoogleFontDependency, JavaScriptDependency
)

GoogleFontDependency(
    label=_('Lato font'), module=__name__, name='lato',
    url='https://fonts.googleapis.com/css?family=Lato:400,700,400italic'
)
JavaScriptDependency(
    label=_('Bootstrap'), module=__name__, name='bootstrap',
    version_string='=3.4.1'
)
JavaScriptDependency(
    label=_('Bootswatch'), module=__name__, name='bootswatch',
    replace_list=[
        {
            'filename_pattern': 'bootstrap.*.css',
            'content_patterns': [
                {
                    'search': '"https://fonts.googleapis.com/css?family=Lato:400,700,400italic"',
                    'replace': '../../../google_fonts/lato/import.css',
                }
            ]
        }
    ], version_string='=3.4.1'
)
JavaScriptDependency(
    label=_('Fancybox'), module=__name__, name='@fancyapps/fancybox',
    version_string='=3.2.5'
)
JavaScriptDependency(
    label=_('FontAwesome'), module=__name__,
    name='@fortawesome/fontawesome-free', version_string='=5.6.3'
)
JavaScriptDependency(
    label=_('jQuery'), module=__name__, name='jquery', version_string='=3.4.1'
)
JavaScriptDependency(
    label=_('JQuery Form'), module=__name__, name='jquery-form',
    version_string='=4.2.2'
)
JavaScriptDependency(
    label=_('jQuery Lazy Load'), module=__name__, name='jquery-lazyload',
    version_string='=1.9.3'
)
JavaScriptDependency(
    label=_('JQuery Match Height'), module=__name__, name='jquery-match-height',
    version_string='=0.7.2'
)
JavaScriptDependency(
    label=_('Select 2'), module=__name__, name='select2',
    version_string='=4.0.3'
)
JavaScriptDependency(
    label=_('Toastr'), module=__name__, name='toastr', version_string='=2.1.4'
)
JavaScriptDependency(
    label=_('URI.js'), module=__name__, name='urijs', version_string='=1.19.1'
)



from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('documents', '0040_auto_20170725_1111'),
        ('ocr', '0005_auto_20170630_1846'),
    ]

    operations = [
        migrations.CreateModel(
            name='DocumentPageOCRContent',
            fields=[
                (
                    'id', models.AutoField(
                        auto_created=True, primary_key=True, serialize=False,
                        verbose_name='ID'
                    )
                ),
                (
                    'content', models.TextField(
                        blank=True, verbose_name='Content'
                    )
                ),
                (
                    'document_page', models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='ocr', to='documents.DocumentPage',
                        verbose_name='Document page'
                    )
                ),
            ],
            options={
                'verbose_name': 'Document page OCR content',
                'verbose_name_plural': 'Document pages OCR contents',
            },
        ),
        migrations.RemoveField(
            model_name='documentpagecontent',
            name='document_page',
        ),
        migrations.AlterModelOptions(
            name='documentversionocrerror',
            options={
                'ordering': ('datetime_submitted',),
                'verbose_name': 'Document version OCR error',
                'verbose_name_plural': 'Document version OCR errors'
            },
        ),
        migrations.AlterField(
            model_name='documentversionocrerror',
            name='datetime_submitted',
            field=models.DateTimeField(
                auto_now_add=True, db_index=True,
                verbose_name='Date time submitted'
            ),
        ),
        migrations.DeleteModel(
            name='DocumentPageContent',
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

TEST_PERMISSION_NAMESPACE_NAME = 'test namespace name'
TEST_PERMISSION_NAMESPACE_TEXT = 'test namespace text'
TEST_PERMISSION_NAME = 'test permission name'
TEST_PERMISSION_LABEL = 'test permission label'
TEST_LINK_TEXT = 'test link text'
TEST_MENU_NAME = 'menu test'
TEST_QUERYSTRING_ONE_KEY = 'key1=value1'
TEST_QUERYSTRING_TWO_KEYS = 'key1=value1&key2=value2'
TEST_SUBMENU_NAME = 'submenu test'
TEST_UNICODE_STRING = 'úñí©óðé'
TEST_URL = 'test-URL'



from __future__ import absolute_import


class DatabaseWarning(UserWarning):
    """
    Warning when using unsupported database backends
    """


class DeprecationWarning(UserWarning):
    """
    Warning when a feature or interface has been deprecated
    """


class InterfaceWarning(UserWarning):
    """
    Warning when using obsolete internal interfaces
    """



from __future__ import absolute_import, unicode_literals

from django.http import Http404, HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.template import RequestContext
from django.urls import reverse
from django.utils.translation import ugettext_lazy as _

from mayan.apps.common.generics import (
    FormView, SingleObjectCreateView, SingleObjectDeleteView,
    SingleObjectDynamicFormCreateView, SingleObjectDynamicFormEditView,
    SingleObjectEditView, SingleObjectListView
)
from mayan.apps.common.mixins import ExternalObjectMixin

from ..classes import WorkflowAction
from ..forms import (
    WorkflowActionSelectionForm, WorkflowStateActionDynamicForm,
    WorkflowStateForm
)
from ..icons import icon_workflow_state, icon_workflow_state_action
from ..links import (
    link_workflow_template_state_create,
    link_workflow_template_state_action_selection,
)
from ..models import Workflow, WorkflowState, WorkflowStateAction
from ..permissions import permission_workflow_edit, permission_workflow_view


class WorkflowTemplateStateActionCreateView(SingleObjectDynamicFormCreateView):
    form_class = WorkflowStateActionDynamicForm
    object_permission = permission_workflow_edit

    def get_class(self):
        try:
            return WorkflowAction.get(name=self.kwargs['class_path'])
        except KeyError:
            raise Http404(
                '{} class not found'.format(self.kwargs['class_path'])
            )

    def get_extra_context(self):
        return {
            'navigation_object_list': ('object', 'workflow'),
            'object': self.get_object(),
            'title': _(
                'Create a "%s" workflow action'
            ) % self.get_class().label,
            'workflow': self.get_object().workflow
        }

    def get_form_extra_kwargs(self):
        return {
            'request': self.request,
            'action_path': self.kwargs['class_path']
        }

    def get_form_schema(self):
        return self.get_class()().get_form_schema(request=self.request)

    def get_instance_extra_data(self):
        return {
            'action_path': self.kwargs['class_path'],
            'state': self.get_object()
        }

    def get_object(self):
        return get_object_or_404(klass=WorkflowState, pk=self.kwargs['pk'])

    def get_post_action_redirect(self):
        return reverse(
            viewname='document_states:workflow_template_state_action_list',
            kwargs={'pk': self.get_object().pk}
        )


class WorkflowTemplateStateActionDeleteView(SingleObjectDeleteView):
    model = WorkflowStateAction
    object_permission = permission_workflow_edit

    def get_extra_context(self):
        return {
            'navigation_object_list': (
                'object', 'workflow_state', 'workflow'
            ),
            'object': self.get_object(),
            'title': _('Delete workflow state action: %s') % self.get_object(),
            'workflow': self.get_object().state.workflow,
            'workflow_state': self.get_object().state,
        }

    def get_post_action_redirect(self):
        return reverse(
            viewname='document_states:workflow_template_state_action_list',
            kwargs={'pk': self.get_object().state.pk}
        )


class WorkflowTemplateStateActionEditView(SingleObjectDynamicFormEditView):
    form_class = WorkflowStateActionDynamicForm
    model = WorkflowStateAction
    object_permission = permission_workflow_edit

    def get_extra_context(self):
        return {
            'navigation_object_list': (
                'object', 'workflow_state', 'workflow'
            ),
            'object': self.get_object(),
            'title': _('Edit workflow state action: %s') % self.get_object(),
            'workflow': self.get_object().state.workflow,
            'workflow_state': self.get_object().state,
        }

    def get_form_extra_kwargs(self):
        return {
            'request': self.request,
            'action_path': self.get_object().action_path,
        }

    def get_form_schema(self):
        return self.get_object().get_class_instance().get_form_schema(
            request=self.request
        )

    def get_post_action_redirect(self):
        return reverse(
            viewname='document_states:workflow_template_state_action_list',
            kwargs={'pk': self.get_object().state.pk}
        )


class WorkflowTemplateStateActionListView(SingleObjectListView):
    object_permission = permission_workflow_edit

    def get_extra_context(self):
        return {
            'hide_object': True,
            'navigation_object_list': ('object', 'workflow'),
            'no_results_icon': icon_workflow_state_action,
            'no_results_main_link': link_workflow_template_state_action_selection.resolve(
                context=RequestContext(
                    request=self.request, dict_={
                        'object': self.get_workflow_state()
                    }
                )
            ),
            'no_results_text': _(
                'Workflow state actions are macros that get executed when '
                'documents enters or leaves the state in which they reside.'
            ),
            'no_results_title': _(
                'There are no actions for this workflow state'
            ),
            'object': self.get_workflow_state(),
            'title': _(
                'Actions for workflow state: %s'
            ) % self.get_workflow_state(),
            'workflow': self.get_workflow_state().workflow,
        }

    def get_form_schema(self):
        return {'fields': self.get_class().fields}

    def get_source_queryset(self):
        return self.get_workflow_state().actions.all()

    def get_workflow_state(self):
        return get_object_or_404(klass=WorkflowState, pk=self.kwargs['pk'])


class WorkflowTemplateStateActionSelectionView(FormView):
    form_class = WorkflowActionSelectionForm
    view_permission = permission_workflow_edit

    def form_valid(self, form):
        klass = form.cleaned_data['klass']
        return HttpResponseRedirect(
            redirect_to=reverse(
                viewname='document_states:workflow_template_state_action_create',
                kwargs={'pk': self.get_object().pk, 'class_path': klass}
            )
        )

    def get_extra_context(self):
        return {
            'navigation_object_list': (
                'object', 'workflow'
            ),
            'object': self.get_object(),
            'title': _('New workflow state action selection'),
            'workflow': self.get_object().workflow,
        }

    def get_object(self):
        return get_object_or_404(klass=WorkflowState, pk=self.kwargs['pk'])


class WorkflowTemplateStateCreateView(ExternalObjectMixin, SingleObjectCreateView):
    external_object_class = Workflow
    external_object_permission = permission_workflow_edit
    external_object_pk_url_kwarg = 'pk'
    form_class = WorkflowStateForm

    def get_extra_context(self):
        return {
            'object': self.external_object,
            'title': _(
                'Create states for workflow: %s'
            ) % self.external_object,
            'workflow': self.external_object
        }

    def get_instance_extra_data(self):
        return {'workflow': self.external_object}

    def get_source_queryset(self):
        return self.external_object.states.all()

    def get_success_url(self):
        return reverse(
            viewname='document_states:workflow_template_state_list',
            kwargs={'pk': self.kwargs['pk']}
        )


class WorkflowTemplateStateDeleteView(SingleObjectDeleteView):
    model = WorkflowState
    object_permission = permission_workflow_edit
    pk_url_kwarg = 'pk'

    def get_extra_context(self):
        return {
            'navigation_object_list': ('object', 'workflow'),
            'object': self.get_object(),
            'title': _(
                'Delete workflow state: %s?'
            ) % self.object,
            'workflow': self.get_object().workflow,
        }

    def get_success_url(self):
        return reverse(
            viewname='document_states:workflow_template_state_list',
            kwargs={'pk': self.get_object().workflow.pk}
        )


class WorkflowTemplateStateEditView(SingleObjectEditView):
    form_class = WorkflowStateForm
    model = WorkflowState
    object_permission = permission_workflow_edit
    pk_url_kwarg = 'pk'

    def get_extra_context(self):
        return {
            'navigation_object_list': ('object', 'workflow'),
            'object': self.get_object(),
            'title': _(
                'Edit workflow state: %s'
            ) % self.object,
            'workflow': self.get_object().workflow,
        }

    def get_success_url(self):
        return reverse(
            viewname='document_states:workflow_template_state_list',
            kwargs={'pk': self.get_object().workflow.pk}
        )


class WorkflowTemplateStateListView(ExternalObjectMixin, SingleObjectListView):
    external_object_class = Workflow
    external_object_permission = permission_workflow_view
    external_object_pk_url_kwarg = 'pk'
    object_permission = permission_workflow_view

    def get_extra_context(self):
        return {
            'hide_object': True,
            'no_results_icon': icon_workflow_state,
            'no_results_main_link': link_workflow_template_state_create.resolve(
                context=RequestContext(
                    self.request, {'workflow': self.external_object}
                )
            ),
            'no_results_text': _(
                'Create states and link them using transitions.'
            ),
            'no_results_title': _(
                'This workflow doesn\'t have any states'
            ),
            'object': self.external_object,
            'title': _('States of workflow: %s') % self.external_object,
            'workflow': self.external_object
        }

    def get_source_queryset(self):
        return self.external_object.states.all()



from __future__ import absolute_import, unicode_literals

import logging

from django.apps import apps
from django.db import models, transaction
from django.urls import reverse
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from mayan.apps.acls.models import AccessControlList
from mayan.apps.common.literals import TIME_DELTA_UNIT_CHOICES

from ..events import event_document_type_created, event_document_type_edited
from ..literals import DEFAULT_DELETE_PERIOD, DEFAULT_DELETE_TIME_UNIT
from ..managers import DocumentTypeManager
from ..permissions import permission_document_view
from ..settings import setting_language

__all__ = ('DocumentType', 'DocumentTypeFilename')
logger = logging.getLogger(__name__)


@python_2_unicode_compatible
class DocumentType(models.Model):
    """
    Define document types or classes to which a specific set of
    properties can be attached
    """
    label = models.CharField(
        help_text=_('The name of the document type.'), max_length=96,
        unique=True, verbose_name=_('Label')
    )
    trash_time_period = models.PositiveIntegerField(
        blank=True, help_text=_(
            'Amount of time after which documents of this type will be '
            'moved to the trash.'
        ), null=True, verbose_name=_('Trash time period')
    )
    trash_time_unit = models.CharField(
        blank=True, choices=TIME_DELTA_UNIT_CHOICES, null=True, max_length=8,
        verbose_name=_('Trash time unit')
    )
    delete_time_period = models.PositiveIntegerField(
        blank=True, default=DEFAULT_DELETE_PERIOD, help_text=_(
            'Amount of time after which documents of this type in the trash '
            'will be deleted.'
        ), null=True, verbose_name=_('Delete time period')
    )
    delete_time_unit = models.CharField(
        blank=True, choices=TIME_DELTA_UNIT_CHOICES,
        default=DEFAULT_DELETE_TIME_UNIT, max_length=8, null=True,
        verbose_name=_('Delete time unit')
    )

    objects = DocumentTypeManager()

    class Meta:
        ordering = ('label',)
        verbose_name = _('Document type')
        verbose_name_plural = _('Documents types')

    def __str__(self):
        return self.label

    def delete(self, *args, **kwargs):
        Document = apps.get_model(app_label='documents', model_name='Document')

        for document in Document.passthrough.filter(document_type=self):
            document.delete(to_trash=False)

        return super(DocumentType, self).delete(*args, **kwargs)

    @property
    def deleted_documents(self):
        DeletedDocument = apps.get_model(
            app_label='documents', model_name='DeletedDocument'
        )

        return DeletedDocument.objects.filter(document_type=self)

    def get_absolute_url(self):
        return reverse(
            viewname='documents:document_type_document_list', kwargs={
                'pk': self.pk
            }
        )

    def get_document_count(self, user):
        queryset = AccessControlList.objects.restrict_queryset(
            permission=permission_document_view, queryset=self.documents,
            user=user
        )

        return queryset.count()

    def natural_key(self):
        return (self.label,)

    def new_document(self, file_object, label=None, description=None, language=None, _user=None):
        try:
            with transaction.atomic():
                document = self.documents.create(
                    description=description or '',
                    label=label or file_object.name,
                    language=language or setting_language.value
                )
                document.save(_user=_user)

                document.new_version(file_object=file_object, _user=_user)
                return document
        except Exception as exception:
            logger.critical(
                'Unexpected exception while trying to create new document '
                '"%s" from document type "%s"; %s',
                label or file_object.name, self, exception
            )
            raise

    def save(self, *args, **kwargs):
        user = kwargs.pop('_user', None)
        created = not self.pk

        result = super(DocumentType, self).save(*args, **kwargs)

        if created:
            event_document_type_created.commit(
                actor=user, target=self
            )
        else:
            event_document_type_edited.commit(
                actor=user, target=self
            )

        return result


@python_2_unicode_compatible
class DocumentTypeFilename(models.Model):
    """
    List of labels available to a specific document type for the
    quick rename functionality
    """
    document_type = models.ForeignKey(
        on_delete=models.CASCADE, related_name='filenames', to=DocumentType,
        verbose_name=_('Document type')
    )
    filename = models.CharField(
        db_index=True, max_length=128, verbose_name=_('Label')
    )
    enabled = models.BooleanField(default=True, verbose_name=_('Enabled'))

    class Meta:
        ordering = ('filename',)
        unique_together = ('document_type', 'filename')
        verbose_name = _('Quick label')
        verbose_name_plural = _('Quick labels')

    def __str__(self):
        return self.filename



from __future__ import unicode_literals

from django.template import Library

register = Library()


@register.simple_tag
def documents_get_api_image_url(obj, **kwargs):
    return obj.get_api_image_url(**kwargs)



from __future__ import unicode_literals

import logging

from django.conf import settings
from django.db import models, transaction
from django.urls import reverse
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from mayan.apps.documents.models import Document

from .events import (
    event_document_comment_created, event_document_comment_deleted,
    event_document_comment_edited
)

logger = logging.getLogger(__name__)


@python_2_unicode_compatible
class Comment(models.Model):
    """
    Model to store one comment per document per user per date & time.
    """
    document = models.ForeignKey(
        db_index=True, on_delete=models.CASCADE, related_name='comments',
        to=Document, verbose_name=_('Document')
    )
    user = models.ForeignKey(
        editable=False, on_delete=models.CASCADE, related_name='comments',
        to=settings.AUTH_USER_MODEL, verbose_name=_('User'),
    )
    # Translators: Comment here is a noun and refers to the actual text stored
    comment = models.TextField(verbose_name=_('Comment'))
    submit_date = models.DateTimeField(
        auto_now_add=True, db_index=True,
        verbose_name=_('Date time submitted')
    )

    class Meta:
        get_latest_by = 'submit_date'
        ordering = ('-submit_date',)
        verbose_name = _('Comment')
        verbose_name_plural = _('Comments')

    def __str__(self):
        return self.comment

    def delete(self, *args, **kwargs):
        _user = kwargs.pop('_user', None)
        with transaction.atomic():
            super(Comment, self).delete(*args, **kwargs)
            event_document_comment_deleted.commit(
                actor=_user, target=self.document
            )

    def get_absolute_url(self):
        return reverse(
            viewname='comments:comment_details', kwargs={'pk': self.pk}
        )

    def get_user_label(self):
        if self.user.get_full_name():
            return self.user.get_full_name()
        else:
            return self.user.username
    get_user_label.short_description = _('User')

    def save(self, *args, **kwargs):
        _user = kwargs.pop('_user', None) or self.user
        created = not self.pk

        with transaction.atomic():
            super(Comment, self).save(*args, **kwargs)
            if created:
                event_document_comment_created.commit(
                    action_object=self.document, actor=_user, target=self,
                )
            else:
                event_document_comment_edited.commit(
                    action_object=self.document, actor=_user, target=self,
                )



from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('checkouts', '0004_auto_20150617_0330'),
    ]

    operations = [
        migrations.AlterField(
            model_name='documentcheckout',
            name='document',
            field=models.OneToOneField(
                verbose_name='Document', to='documents.Document'
            ),
        ),
    ]



from __future__ import unicode_literals

from django.db import migrations, models
import mayan.apps.document_signatures.models
import mayan.apps.storage.classes


class Migration(migrations.Migration):

    dependencies = [
        ('document_signatures', '0009_auto_20190711_0544'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='signaturebasemodel',
            options={
                'ordering': ('pk',),
                'verbose_name': 'Document version signature',
                'verbose_name_plural': 'Document version signatures'
            },
        ),
        migrations.AlterField(
            model_name='detachedsignature',
            name='signature_file',
            field=models.FileField(
                blank=True, help_text='Signature file previously generated.',
                null=True,
                storage=mayan.apps.storage.classes.FakeStorageSubclass(),
                upload_to=mayan.apps.document_signatures.models.upload_to,
                verbose_name='Signature file'
            ),
        ),
        migrations.AlterField(
            model_name='signaturebasemodel',
            name='key_id',
            field=models.CharField(
                help_text='ID of the key that will be used to sign the '
                'document.', max_length=40, verbose_name='Key ID'
            ),
        ),
    ]



from __future__ import absolute_import, unicode_literals

from mayan.apps.appearance.classes import Icon

icon_document_content = Icon(driver_name='fontawesome', symbol='font')
icon_document_content_delete = Icon(
    driver_name='fontawesome-dual',
    primary_symbol='font',
    secondary_symbol='minus'
)
icon_document_parsing_errors_list = Icon(
    driver_name='fontawesome', symbol='font'
)
icon_document_content_download = Icon(
    driver_name='fontawesome', symbol='font'
)
icon_document_multiple_submit = Icon(
    driver_name='fontawesome', symbol='font'
)
icon_document_submit = Icon(
    driver_name='fontawesome', symbol='font'
)
icon_document_type_parsing_settings = Icon(
    driver_name='fontawesome', symbol='font'
)
icon_document_type_submit = Icon(
    driver_name='fontawesome', symbol='font'
)
icon_link_error_list = Icon(
    driver_name='fontawesome', symbol='font'
)



from __future__ import absolute_import, unicode_literals

from rest_framework import generics

from .filters import MayanObjectPermissionsFilter
from .permissions import MayanPermission


class GenericAPIView(generics.GenericAPIView):
    filter_backends = (MayanObjectPermissionsFilter,)
    permission_classes = (MayanPermission,)


class ListAPIView(generics.ListAPIView):
    """
    requires:
        object_permission = {'GET': ...}
    """
    filter_backends = (MayanObjectPermissionsFilter,)


class ListCreateAPIView(generics.ListCreateAPIView):
    """
    requires:
        object_permission = {'GET': ...}
        view_permission = {'POST': ...}
    """
    filter_backends = (MayanObjectPermissionsFilter,)
    permission_classes = (MayanPermission,)


class RetrieveAPIView(generics.RetrieveAPIView):
    """
    requires:
        object_permission = {
            'GET': ...,
        }
    """
    filter_backends = (MayanObjectPermissionsFilter,)


class RetrieveDestroyAPIView(generics.RetrieveDestroyAPIView):
    """
    requires:
        object_permission = {
            'DELETE': ...,
            'GET': ...,
        }
    """
    filter_backends = (MayanObjectPermissionsFilter,)


class RetrieveUpdateAPIView(generics.RetrieveUpdateAPIView):
    """
    requires:
        object_permission = {
            'GET': ...,
            'PATCH': ...,
            'PUT': ...
        }
    """
    filter_backends = (MayanObjectPermissionsFilter,)


class RetrieveUpdateDestroyAPIView(generics.RetrieveUpdateDestroyAPIView):
    """
    requires:
        object_permission = {
            'DELETE': ...,
            'GET': ...,
            'PATCH': ...,
            'PUT': ...
        }
    """
    filter_backends = (MayanObjectPermissionsFilter,)



from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('sources', '0002_auto_20150608_1902'),
    ]

    operations = [
        migrations.CreateModel(
            name='SourceLog',
            fields=[
                (
                    'id', models.AutoField(
                        verbose_name='ID', serialize=False, auto_created=True,
                        primary_key=True
                    )
                ),
                (
                    'datetime', models.DateTimeField(
                        auto_now_add=True, verbose_name='Date time'
                    )
                ),
                (
                    'message', models.TextField(
                        verbose_name='Message', editable=False, blank=True
                    )
                ),
                (
                    'source', models.ForeignKey(
                        related_name='logs', verbose_name='Source',
                        to='sources.Source'
                    )
                ),
            ],
            options={
                'ordering': ['-datetime'],
                'get_latest_by': 'datetime',
                'verbose_name': 'Log entry',
                'verbose_name_plural': 'Log entries',
            },
            bases=(models.Model,),
        ),
    ]



from __future__ import absolute_import, unicode_literals

from django import forms
from django.utils.translation import ugettext_lazy as _

from .fields import TemplateField


class DocumentTemplateSandboxForm(forms.Form):
    result = forms.CharField(
        help_text=_('Resulting text from the evaluated template.'),
        label=_('Result'), required=False, widget=forms.widgets.Textarea(
            attrs={'readonly': 'readonly', 'rows': 5}
        )
    )

    def __init__(self, *args, **kwargs):
        self.model = kwargs.pop('model')
        self.model_variable = kwargs.pop('model_variable')
        super(DocumentTemplateSandboxForm, self).__init__(*args, **kwargs)
        self.fields['template'] = TemplateField(
            initial_help_text=_('The template string to be evaluated.'),
            label=_('Template'), model=self.model,
            model_variable=self.model_variable, required=True
        )
        self.order_fields(field_order=('template', 'result'))



from __future__ import unicode_literals

from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group
from django.shortcuts import get_object_or_404

from mayan.apps.acls.models import AccessControlList
from mayan.apps.rest_api import generics

from .permissions import (
    permission_group_create, permission_group_delete, permission_group_edit,
    permission_group_view, permission_user_create, permission_user_delete,
    permission_user_edit, permission_user_view
)
from .serializers import (
    GroupSerializer, UserSerializer, UserGroupListSerializer
)


class APICurrentUserView(generics.RetrieveUpdateDestroyAPIView):
    """
    delete: Delete the current user.
    get: Return the details of the current user.
    patch: Partially edit the current user.
    put: Edit the current user.
    """
    serializer_class = UserSerializer

    def get_object(self):
        return self.request.user


class APIGroupListView(generics.ListCreateAPIView):
    """
    get: Returns a list of all the groups.
    post: Create a new group.
    """
    mayan_object_permissions = {'GET': (permission_group_view,)}
    mayan_view_permissions = {'POST': (permission_group_create,)}
    queryset = Group.objects.order_by('id')
    serializer_class = GroupSerializer


class APIGroupView(generics.RetrieveUpdateDestroyAPIView):
    """
    delete: Delete the selected group.
    get: Return the details of the selected group.
    patch: Partially edit the selected group.
    put: Edit the selected group.
    """
    mayan_object_permissions = {
        'GET': (permission_group_view,),
        'PUT': (permission_group_edit,),
        'PATCH': (permission_group_edit,),
        'DELETE': (permission_group_delete,)
    }
    queryset = Group.objects.order_by('id')
    serializer_class = GroupSerializer


class APIUserListView(generics.ListCreateAPIView):
    """
    get: Returns a list of all the users.
    post: Create a new user.
    """
    mayan_object_permissions = {'GET': (permission_user_view,)}
    mayan_view_permissions = {'POST': (permission_user_create,)}
    queryset = get_user_model().objects.all()
    serializer_class = UserSerializer


class APIUserView(generics.RetrieveUpdateDestroyAPIView):
    """
    delete: Delete the selected user.
    get: Return the details of the selected user.
    patch: Partially edit the selected user.
    put: Edit the selected user.
    """
    mayan_object_permissions = {
        'GET': (permission_user_view,),
        'PUT': (permission_user_edit,),
        'PATCH': (permission_user_edit,),
        'DELETE': (permission_user_delete,)
    }
    queryset = get_user_model().objects.all()
    serializer_class = UserSerializer


class APIUserGroupList(generics.ListCreateAPIView):
    """
    get: Returns a list of all the groups to which a user belongs.
    post: Add a user to a list of groups.
    """
    def get_serializer(self, *args, **kwargs):
        if not self.request:
            return None

        return super(APIUserGroupList, self).get_serializer(*args, **kwargs)

    def get_serializer_class(self):
        if self.request.method == 'GET':
            return GroupSerializer
        elif self.request.method == 'POST':
            return UserGroupListSerializer

    def get_serializer_context(self):
        """
        Extra context provided to the serializer class.
        """
        context = super(APIUserGroupList, self).get_serializer_context()
        if self.kwargs:
            context.update(
                {
                    'user': self.get_user(),
                }
            )

        return context

    def get_queryset(self):
        user = self.get_user()

        return AccessControlList.objects.restrict_queryset(
            permission=permission_group_view,
            queryset=user.groups.order_by('id'), user=self.request.user
        )

    def get_user(self):
        if self.request.method == 'GET':
            permission = permission_user_view
        else:
            permission = permission_user_edit

        user = get_object_or_404(klass=get_user_model(), pk=self.kwargs['pk'])

        AccessControlList.objects.check_access(
            obj=user, permissions=(permission,), user=self.request.user
        )
        return user

    def perform_create(self, serializer):
        serializer.save(user=self.get_user(), _user=self.request.user)



from __future__ import unicode_literals

from django.conf.urls import url

from .views import (
    TransformationCreateView, TransformationDeleteView,
    TransformationEditView, TransformationListView, TransformationSelectView
)

urlpatterns = [
    url(
        regex=r'^object/(?P<app_label>[-\w]+)/(?P<model>[-\w]+)/(?P<object_id>\d+)/layers/(?P<layer_name>[-_\w]+)/transformations/$',
        view=TransformationListView.as_view(), name='transformation_list'
    ),
    url(
        regex=r'^object/(?P<app_label>[-\w]+)/(?P<model>[-\w]+)/(?P<object_id>\d+)/layers/(?P<layer_name>[-_\w]+)/transformations/select/$',
        view=TransformationSelectView.as_view(), name='transformation_select'
    ),
    url(
        regex=r'^object/(?P<app_label>[-\w]+)/(?P<model>[-\w]+)/(?P<object_id>\d+)/layers/(?P<layer_name>[-_\w]+)/transformations/(?P<transformation_name>[-_\w]+)/create/$',
        view=TransformationCreateView.as_view(), name='transformation_create'
    ),
    url(
        regex=r'^layers/(?P<layer_name>[-_\w]+)/transformations/(?P<pk>\d+)/delete/$',
        view=TransformationDeleteView.as_view(), name='transformation_delete'
    ),
    url(
        regex=r'^layers/(?P<layer_name>[-_\w]+)/transformations/(?P<pk>\d+)/edit/$',
        view=TransformationEditView.as_view(), name='transformation_edit'
    ),
]



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from mayan.apps.common.queues import queue_tools
from mayan.apps.task_manager.classes import CeleryQueue
from mayan.apps.task_manager.workers import worker_medium

queue_indexing = CeleryQueue(label=_('Indexing'), name='indexing', worker=worker_medium)

queue_indexing.add_task_type(
    label=_('Delete empty index nodes'),
    dotted_path='mayan.apps.document_indexing.tasks.task_delete_empty'
)
queue_indexing.add_task_type(
    label=_('Remove document'),
    dotted_path='mayan.apps.document_indexing.tasks.task_remove_document'
)
queue_indexing.add_task_type(
    label=_('Index document'),
    dotted_path='mayan.apps.document_indexing.tasks.task_index_document'
)
queue_tools.add_task_type(
    label=_('Rebuild index'),
    dotted_path='mayan.apps.document_indexing.tasks.task_rebuild_index'
)



from __future__ import absolute_import, unicode_literals

from django.apps import apps
from django.urls import reverse_lazy
from django.utils.translation import ugettext_lazy as _

from mayan.apps.dashboards.classes import DashboardWidgetNumeric
from mayan.apps.documents.permissions import permission_document_view

from .icons import icon_dashboard_check_outs
from .permissions import permission_document_check_out_detail_view


class DashboardWidgetTotalCheckouts(DashboardWidgetNumeric):
    icon_class = icon_dashboard_check_outs
    label = _('Checked out documents')
    link = reverse_lazy(viewname='checkouts:check_out_list')

    def render(self, request):
        AccessControlList = apps.get_model(
            app_label='acls', model_name='AccessControlList'
        )
        DocumentCheckout = apps.get_model(
            app_label='checkouts', model_name='DocumentCheckout'
        )
        queryset = AccessControlList.objects.restrict_queryset(
            permission=permission_document_check_out_detail_view,
            queryset=DocumentCheckout.objects.checked_out_documents(),
            user=request.user
        )
        queryset = AccessControlList.objects.restrict_queryset(
            permission=permission_document_view, queryset=queryset,
            user=request.user
        )
        self.count = queryset.count()
        return super(DashboardWidgetTotalCheckouts, self).render(request)



from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('documents', '__first__'),
        ('contenttypes', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='DocumentCheckout',
            fields=[
                (
                    'id', models.AutoField(
                        verbose_name='ID', serialize=False, auto_created=True,
                        primary_key=True
                    )
                ),
                (
                    'checkout_datetime',
                    models.DateTimeField(
                        auto_now_add=True,
                        verbose_name='Check out date and time'
                    )
                ),
                (
                    'expiration_datetime',
                    models.DateTimeField(
                        help_text='Amount of time to hold the document '
                        'checked out in minutes.',
                        verbose_name='Check out expiration date and time'
                    )
                ),
                (
                    'user_object_id',
                    models.PositiveIntegerField(null=True, blank=True)
                ),
                (
                    'block_new_version',
                    models.BooleanField(
                        default=True,
                        help_text='Do not allow new version of this document '
                        'to be uploaded.',
                        verbose_name='Block new version upload'
                    )
                ),
                (
                    'document',
                    models.ForeignKey(
                        verbose_name='Document', to='documents.Document',
                        unique=True
                    )
                ),
                (
                    'user_content_type',
                    models.ForeignKey(
                        blank=True, to='contenttypes.ContentType', null=True
                    )
                ),
            ],
            options={
                'verbose_name': 'Document checkout',
                'verbose_name_plural': 'Document checkouts',
            },
            bases=(models.Model,),
        ),
    ]



from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='StatisticResult',
            fields=[
                (
                    'id', models.AutoField(
                        verbose_name='ID', serialize=False, auto_created=True,
                        primary_key=True
                    )
                ),
                ('slug', models.SlugField(verbose_name='Slug')),
                (
                    'datetime', models.DateTimeField(
                        auto_now=True, verbose_name='Date time'
                    )
                ),
                (
                    'serialize_data', models.TextField(
                        verbose_name='Data', blank=True
                    )
                ),
            ],
            options={
                'verbose_name': 'Statistics result',
                'verbose_name_plural': 'Statistics results',
            },
            bases=(models.Model,),
        ),
    ]



from __future__ import unicode_literals

from django import apps
from django.utils.translation import ugettext_lazy as _


class LockManagerApp(apps.AppConfig):
    has_tests = True
    name = 'mayan.apps.lock_manager'
    verbose_name = _('Lock manager')



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from mayan.apps.smart_settings.classes import Namespace

from .literals import DEFAULT_BACKEND, DEFAULT_LOCK_TIMEOUT_VALUE

namespace = Namespace(label=_('Lock manager'), name='lock_manager')

setting_backend = namespace.add_setting(
    default=DEFAULT_BACKEND,
    global_name='LOCK_MANAGER_BACKEND', help_text=_(
        'Path to the class to use when to request and release '
        'resource locks.'
    )
)
setting_backend_arguments = namespace.add_setting(
    global_name='LOCK_MANAGER_BACKEND_ARGUMENTS',
    default={}, help_text=_('Arguments to pass to the LOCK_MANAGER_BACKEND.')
)

setting_default_lock_timeout = namespace.add_setting(
    default=DEFAULT_LOCK_TIMEOUT_VALUE,
    global_name='LOCK_MANAGER_DEFAULT_LOCK_TIMEOUT', help_text=_(
        'Default amount of time in seconds after which a resource '
        'lock will be automatically released.'
    )
)



from __future__ import unicode_literals

from django.apps import apps
from django.core.exceptions import ImproperlyConfigured
from django.db.models import Manager
from django.db.models.query import QuerySet

from rest_framework import serializers
from rest_framework.relations import HyperlinkedIdentityField

from mayan.apps.common.utils import resolve_attribute


class FilteredPrimaryKeyRelatedField(serializers.PrimaryKeyRelatedField):
    def __init__(self, **kwargs):
        self.source_model = kwargs.pop('source_model', None)
        self.source_permission = kwargs.pop('source_permission', None)
        self.source_queryset = kwargs.pop('source_queryset', None)
        self.source_queryset_method = kwargs.pop('source_queryset_method', None)
        super(FilteredPrimaryKeyRelatedField, self).__init__(**kwargs)

    def get_queryset(self):
        AccessControlList = apps.get_model(
            app_label='acls', model_name='AccessControlList'
        )

        if self.source_model:
            queryset = self.source_model._meta.default_manager.all()
        elif self.source_queryset:
            queryset = self.source_queryset
            if isinstance(queryset, (QuerySet, Manager)):
                # Ensure queryset is re-evaluated whenever used.
                queryset = queryset.all()
        else:
            method_name = self.source_queryset_method or 'get_{}_queryset'.format(
                self.field_name
            )
            try:
                queryset = getattr(self.parent, method_name)()
            except AttributeError:
                raise ImproperlyConfigured(
                    'Need to provide a source_model, a '
                    'source_queryset, a source_queryset_method, or '
                    'a method named "%s".' % method_name
                )

        assert 'request' in self.context, (
            "`%s` requires the request in the serializer"
            " context. Add `context={'request': request}` when instantiating "
            "the serializer." % self.__class__.__name__
        )

        request = self.context['request']

        if self.source_permission:
            return AccessControlList.objects.restrict_queryset(
                permission=self.source_permission, queryset=queryset,
                user=request.user
            )
        else:
            return queryset


class MultiKwargHyperlinkedIdentityField(HyperlinkedIdentityField):
    def __init__(self, *args, **kwargs):
        self.view_kwargs = kwargs.pop('view_kwargs', [])
        super(MultiKwargHyperlinkedIdentityField, self).__init__(*args, **kwargs)

    def get_url(self, obj, view_name, request, format):
        """
        Extends HyperlinkedRelatedField to allow passing more than one view
        keyword argument.
        ----
        Given an object, return the URL that hyperlinks to the object.

        May raise a `NoReverseMatch` if the `view_name` and `lookup_field`
        attributes are not configured to correctly match the URL conf.
        """
        # Unsaved objects will not yet have a valid URL.
        if hasattr(obj, 'pk') and obj.pk in (None, ''):
            return None

        kwargs = {}
        for entry in self.view_kwargs:
            kwargs[entry['lookup_url_kwarg']] = resolve_attribute(
                obj=obj, attribute=entry['lookup_field']
            )

        return self.reverse(
            viewname=view_name, kwargs=kwargs, request=request, format=format
        )



from __future__ import unicode_literals

from actstream.models import Action

from mayan.apps.documents.tests.test_models import GenericDocumentTestCase

from ..events import (
    event_ocr_document_content_deleted, event_ocr_document_version_submit,
    event_ocr_document_version_finish
)
from ..models import DocumentPageOCRContent


class OCREventsTestCase(GenericDocumentTestCase):
    def test_document_content_deleted_event(self):
        Action.objects.all().delete()
        DocumentPageOCRContent.objects.delete_content_for(
            document=self.test_document
        )

        # Get the oldest action
        action = Action.objects.order_by('-timestamp').last()

        self.assertEqual(
            action.target, self.test_document
        )
        self.assertEqual(
            action.verb, event_ocr_document_content_deleted.id
        )

    def test_document_version_submit_event(self):
        Action.objects.all().delete()
        self.test_document.submit_for_ocr()

        # Get the oldest action
        action = Action.objects.order_by('-timestamp').last()

        self.assertEqual(
            action.target, self.test_document.latest_version
        )
        self.assertEqual(
            action.verb, event_ocr_document_version_submit.id
        )

    def test_document_version_finish_event(self):
        Action.objects.all().delete()
        self.test_document.submit_for_ocr()

        # Get the most recent action
        action = Action.objects.order_by('-timestamp').first()

        self.assertEqual(
            action.target, self.test_document.latest_version
        )
        self.assertEqual(
            action.verb, event_ocr_document_version_finish.id
        )



from __future__ import unicode_literals


class APIError(Exception):
    """
    Base exception for the API app
    """
    pass


class APIResourcePatternError(APIError):
    """
    Raised when an app tries to override an existing URL regular expression
    pattern
    """
    pass



from __future__ import unicode_literals

import yaml

try:
    from yaml import CSafeLoader as SafeLoader, CSafeDumper as SafeDumper
except ImportError:
    from yaml import SafeLoader, SafeDumper


def yaml_dump(*args, **kwargs):
    defaults = {'Dumper': SafeDumper}
    defaults.update(kwargs)

    return yaml.dump(*args, **defaults)


def yaml_load(*args, **kwargs):
    defaults = {'Loader': SafeLoader}
    defaults.update(kwargs)

    return yaml.load(*args, **defaults)



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from mayan.apps.metadata.api import (
    decode_metadata_from_querystring, save_metadata_list
)
from mayan.apps.metadata.forms import DocumentMetadataFormSet

from mayan.apps.sources.wizards import WizardStep, WizardStepDocumentType


class WizardStepMetadata(WizardStep):
    form_class = DocumentMetadataFormSet
    label = _('Enter document metadata')
    name = 'metadata_entry'
    number = 1

    @classmethod
    def condition(cls, wizard):
        """
        Skip step if document type has no associated metadata
        """
        cleaned_data = wizard.get_cleaned_data_for_step(WizardStepDocumentType.name) or {}

        document_type = cleaned_data.get('document_type')

        if document_type:
            return document_type.metadata.exists()

    @classmethod
    def get_form_initial(cls, wizard):
        initial = []

        step_data = wizard.get_cleaned_data_for_step(WizardStepDocumentType.name)
        if step_data:
            document_type = step_data['document_type']
            for document_type_metadata_type in document_type.metadata.all():
                initial.append(
                    {
                        'document_type': document_type,
                        'metadata_type': document_type_metadata_type.metadata_type,
                    }
                )

        return initial

    @classmethod
    def done(cls, wizard):
        result = {}
        cleaned_data = wizard.get_cleaned_data_for_step(cls.name)
        if cleaned_data:
            for identifier, metadata in enumerate(wizard.get_cleaned_data_for_step(cls.name)):
                if metadata.get('update'):
                    result['metadata%s_id' % identifier] = metadata['id']
                    result['metadata%s_value' % identifier] = metadata['value']

        return result

    @classmethod
    def step_post_upload_process(cls, document, querystring=None):
        metadata_dict_list = decode_metadata_from_querystring(querystring=querystring)
        if metadata_dict_list:
            save_metadata_list(
                metadata_list=metadata_dict_list, document=document,
                create=True
            )


WizardStep.register(step=WizardStepMetadata)



from __future__ import unicode_literals

from django.db import migrations


def operation_move_content_from_documents_to_ocr_app(apps, schema_editor):
    DocumentPage = apps.get_model(
        app_label='documents', model_name='DocumentPage'
    )
    DocumentPageContent = apps.get_model(
        app_label='ocr', model_name='DocumentPageContent'
    )

    for document_page in DocumentPage.objects.using(schema_editor.connection.alias).all():
        DocumentPageContent.objects.using(schema_editor.connection.alias).create(
            document_page=document_page,
            content=document_page.content_old or ''
        )


class Migration(migrations.Migration):

    dependencies = [
        ('ocr', '0002_documentpagecontent'),
    ]

    operations = [
        migrations.RunPython(
            code=operation_move_content_from_documents_to_ocr_app
        ),
    ]



from __future__ import unicode_literals

from django.contrib import admin

from .models import (
    DocumentPageOCRContent, DocumentTypeSettings, DocumentVersionOCRError
)


@admin.register(DocumentPageOCRContent)
class DocumentPageOCRContentAdmin(admin.ModelAdmin):
    list_display = ('document_page',)


@admin.register(DocumentTypeSettings)
class DocumentTypeSettingsAdmin(admin.ModelAdmin):
    list_display = ('document_type', 'auto_ocr')


@admin.register(DocumentVersionOCRError)
class DocumentVersionOCRErrorAdmin(admin.ModelAdmin):
    list_display = ('document_version', 'datetime_submitted')
    readonly_fields = ('document_version', 'datetime_submitted', 'result')



from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('documents', '0026_auto_20150729_2140'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='documenttypefilename',
            options={
                'ordering': ('filename',),
                'verbose_name': 'Quick rename template',
                'verbose_name_plural': 'Quick rename templates'
            },
        ),
        migrations.AlterField(
            model_name='document',
            name='is_stub',
            field=models.BooleanField(
                default=True, help_text='A document stub is a document with '
                'an entry on the database but no file uploaded. This could '
                'be an interrupted upload or a deferred upload via the API.',
                verbose_name='Is stub?', editable=False
            ),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='documenttypefilename',
            name='filename',
            field=models.CharField(
                max_length=128, verbose_name='Label', db_index=True
            ),
            preserve_default=True,
        ),
    ]



from __future__ import absolute_import, unicode_literals

from django import forms
from django.utils.translation import ugettext_lazy as _

from mayan.apps.acls.models import AccessControlList

from ..models import DocumentType, DocumentTypeFilename

__all__ = ('DocumentTypeFilteredSelectForm', 'DocumentTypeFilenameForm_create')


class DocumentTypeFilteredSelectForm(forms.Form):
    """
    Form to select the document type of a document to be created. This form
    is meant to be reused and reconfigured by other apps. Example: Used
    as form #1 in the document creation wizard.
    """
    def __init__(self, *args, **kwargs):
        help_text = kwargs.pop('help_text', None)
        if kwargs.pop('allow_multiple', False):
            extra_kwargs = {}
            field_class = forms.ModelMultipleChoiceField
            widget_class = forms.widgets.SelectMultiple
        else:
            extra_kwargs = {'empty_label': None}
            field_class = forms.ModelChoiceField
            widget_class = forms.widgets.Select

        permission = kwargs.pop('permission', None)
        user = kwargs.pop('user', None)

        super(DocumentTypeFilteredSelectForm, self).__init__(*args, **kwargs)

        queryset = DocumentType.objects.all()
        if permission:
            queryset = AccessControlList.objects.restrict_queryset(
                permission=permission, queryset=queryset, user=user
            )

        self.fields['document_type'] = field_class(
            help_text=help_text, label=_('Document type'),
            queryset=queryset, required=True,
            widget=widget_class(attrs={'class': 'select2', 'size': 10}),
            **extra_kwargs
        )


class DocumentTypeFilenameForm_create(forms.ModelForm):
    """
    Model class form to create a new document type filename
    """
    class Meta:
        fields = ('filename',)
        model = DocumentTypeFilename



from __future__ import absolute_import, unicode_literals

from django.core.exceptions import PermissionDenied
from django.db import models

from mayan.apps.common.tests.base import BaseTestCase

from ..classes import ModelPermission
from ..models import AccessControlList

from .mixins import ACLTestMixin


class PermissionTestCase(ACLTestMixin, BaseTestCase):
    auto_create_test_object = False

    def test_check_access_without_permissions(self):
        self._setup_test_object()

        with self.assertRaises(PermissionDenied):
            AccessControlList.objects.check_access(
                obj=self.test_object,
                permissions=(self.test_permission,),
                user=self._test_case_user,
            )

    def test_filtering_without_permissions(self):
        self._setup_test_object()

        self.assertEqual(
            AccessControlList.objects.restrict_queryset(
                permission=self.test_permission,
                queryset=self.test_object._meta.model._default_manager.all(),
                user=self._test_case_user
            ).count(), 0
        )

    def test_check_access_with_acl(self):
        self._setup_test_object()

        self.grant_access(
            obj=self.test_object, permission=self.test_permission
        )

        try:
            AccessControlList.objects.check_access(
                obj=self.test_object, permissions=(self.test_permission,),
                user=self._test_case_user,
            )
        except PermissionDenied:
            self.fail('PermissionDenied exception was not expected.')

    def test_filtering_with_permissions(self):
        self._setup_test_object()

        self.grant_access(
            obj=self.test_object, permission=self.test_permission
        )

        self.assertTrue(
            self.test_object in AccessControlList.objects.restrict_queryset(
                permission=self.test_permission,
                queryset=self.test_object._meta.model._default_manager.all(),
                user=self._test_case_user
            )
        )

    def _setup_child_parent_test_objects(self):
        self._create_test_permission()
        self.TestModelParent = self._create_test_model(
            model_name='TestModelParent'
        )
        self.TestModelChild = self._create_test_model(
            fields={
                'parent': models.ForeignKey(
                    on_delete=models.CASCADE, related_name='children',
                    to='TestModelParent',
                )
            }, model_name='TestModelChild'
        )

        ModelPermission.register(
            model=self.TestModelParent, permissions=(
                self.test_permission,
            )
        )
        ModelPermission.register(
            model=self.TestModelChild, permissions=(
                self.test_permission,
            )
        )
        ModelPermission.register_inheritance(
            model=self.TestModelChild, related='parent',
        )

        self.test_object_parent = self.TestModelParent.objects.create()
        self.test_object_child = self.TestModelChild.objects.create(
            parent=self.test_object_parent
        )

    def test_check_access_with_inherited_acl(self):
        self._setup_child_parent_test_objects()

        self.grant_access(
            obj=self.test_object_parent, permission=self.test_permission
        )

        try:
            AccessControlList.objects.check_access(
                obj=self.test_object_child,
                permissions=(self.test_permission,),
                user=self._test_case_user
            )
        except PermissionDenied:
            self.fail('PermissionDenied exception was not expected.')

    def test_check_access_with_inherited_acl_and_local_acl(self):
        self._setup_child_parent_test_objects()

        self.grant_access(
            obj=self.test_object_parent, permission=self.test_permission
        )
        self.grant_access(
            obj=self.test_object_child, permission=self.test_permission
        )

        try:
            AccessControlList.objects.check_access(
                obj=self.test_object_child,
                permissions=(self.test_permission,),
                user=self._test_case_user
            )
        except PermissionDenied:
            self.fail('PermissionDenied exception was not expected.')

    def test_filtering_with_inherited_permissions(self):
        self._setup_child_parent_test_objects()

        self.grant_access(
            obj=self.test_object_parent, permission=self.test_permission
        )

        result = AccessControlList.objects.restrict_queryset(
            permission=self.test_permission,
            queryset=self.test_object_child._meta.model._default_manager.all(),
            user=self._test_case_user
        )
        self.assertTrue(self.test_object_child in result)

    def test_filtering_with_inherited_permissions_and_local_acl(self):
        self._setup_child_parent_test_objects()

        self.grant_permission(permission=self.test_permission)
        self.grant_access(
            obj=self.test_object_parent, permission=self.test_permission
        )
        self.grant_access(
            obj=self.test_object_child, permission=self.test_permission
        )

        result = AccessControlList.objects.restrict_queryset(
            permission=self.test_permission,
            queryset=self.test_object_child._meta.model._default_manager.all(),
            user=self._test_case_user,
        )
        self.assertTrue(self.test_object_child in result)


class InheritedPermissionTestCase(ACLTestMixin, BaseTestCase):
    def test_retrieve_inherited_role_permission_not_model_applicable(self):
        self.TestModel = self._create_test_model()
        self.test_object = self.TestModel.objects.create()
        self._create_test_acl()
        self._create_test_permission()

        self.test_role.grant(permission=self.test_permission)

        queryset = AccessControlList.objects.get_inherited_permissions(
            obj=self.test_object, role=self.test_role
        )
        self.assertTrue(
            self.test_permission.stored_permission not in queryset
        )

    def test_retrieve_inherited_role_permission_model_applicable(self):
        self.TestModel = self._create_test_model()
        self.test_object = self.TestModel.objects.create()
        self._create_test_acl()
        self._create_test_permission()

        ModelPermission.register(
            model=self.test_object._meta.model, permissions=(
                self.test_permission,
            )
        )
        self.test_role.grant(permission=self.test_permission)

        queryset = AccessControlList.objects.get_inherited_permissions(
            obj=self.test_object, role=self.test_role
        )
        self.assertTrue(self.test_permission.stored_permission in queryset)

    def test_retrieve_inherited_related_parent_child_permission(self):
        self._create_test_permission()

        self.TestModelParent = self._create_test_model(
            model_name='TestModelParent'
        )
        self.TestModelChild = self._create_test_model(
            fields={
                'parent': models.ForeignKey(
                    on_delete=models.CASCADE, related_name='children',
                    to='TestModelParent',
                )
            }, model_name='TestModelChild'
        )

        ModelPermission.register(
            model=self.TestModelParent, permissions=(
                self.test_permission,
            )
        )
        ModelPermission.register(
            model=self.TestModelChild, permissions=(
                self.test_permission,
            )
        )
        ModelPermission.register_inheritance(
            model=self.TestModelChild, related='parent',
        )

        parent = self.TestModelParent.objects.create()
        child = self.TestModelChild.objects.create(parent=parent)

        AccessControlList.objects.grant(
            obj=parent, permission=self.test_permission, role=self.test_role
        )
        queryset = AccessControlList.objects.get_inherited_permissions(
            obj=child, role=self.test_role
        )

        self.assertTrue(self.test_permission.stored_permission in queryset)

    def test_retrieve_inherited_related_grandparent_parent_child_permission(
        self
    ):
        self._create_test_permission()

        self.TestModelGrandParent = self._create_test_model(
            model_name='TestModelGrandParent'
        )
        self.TestModelParent = self._create_test_model(
            fields={
                'parent': models.ForeignKey(
                    on_delete=models.CASCADE, related_name='children',
                    to='TestModelGrandParent',
                )
            }, model_name='TestModelParent'
        )
        self.TestModelChild = self._create_test_model(
            fields={
                'parent': models.ForeignKey(
                    on_delete=models.CASCADE, related_name='children',
                    to='TestModelParent',
                )
            }, model_name='TestModelChild'
        )

        ModelPermission.register(
            model=self.TestModelGrandParent, permissions=(
                self.test_permission,
            )
        )
        ModelPermission.register(
            model=self.TestModelParent, permissions=(
                self.test_permission,
            )
        )
        ModelPermission.register(
            model=self.TestModelChild, permissions=(
                self.test_permission,
            )
        )
        ModelPermission.register_inheritance(
            model=self.TestModelChild, related='parent',
        )
        ModelPermission.register_inheritance(
            model=self.TestModelParent, related='parent',
        )

        grandparent = self.TestModelGrandParent.objects.create()
        parent = self.TestModelParent.objects.create(parent=grandparent)
        child = self.TestModelChild.objects.create(parent=parent)

        AccessControlList.objects.grant(
            obj=grandparent, permission=self.test_permission,
            role=self.test_role
        )

        queryset = AccessControlList.objects.get_inherited_permissions(
            obj=child, role=self.test_role
        )

        self.assertTrue(self.test_permission.stored_permission in queryset)


class ProxyModelPermissionTestCase(ACLTestMixin, BaseTestCase):
    def test_proxy_model_filtering_no_permission(self):
        self._setup_test_object_base()
        self._setup_test_object_proxy()

        proxy_object = self.TestModelProxy.objects.get(pk=self.test_object.pk)

        self.assertFalse(
            proxy_object in AccessControlList.objects.restrict_queryset(
                permission=self.test_permission,
                queryset=self.TestModelProxy.objects.all(),
                user=self._test_case_user
            )
        )

    def test_proxy_model_filtering_with_access(self):
        self._setup_test_object_base()
        self._setup_test_object_proxy()

        self.grant_access(
            obj=self.test_object, permission=self.test_permission
        )

        proxy_object = self.TestModelProxy.objects.get(pk=self.test_object.pk)

        self.assertTrue(
            proxy_object in AccessControlList.objects.restrict_queryset(
                permission=self.test_permission,
                queryset=self.TestModelProxy.objects.all(),
                user=self._test_case_user
            )
        )

    def test_proxy_model_inheritance_with_access(self):
        self._create_test_permission()

        self.TestModelParent = self._create_test_model(
            model_name='TestModelParent'
        )
        self.TestModelChild = self._create_test_model(
            fields={
                'parent': models.ForeignKey(
                    on_delete=models.CASCADE, related_name='children',
                    to='TestModelParent',
                )
            }, model_name='TestModelChild'
        )
        self.TestModelProxy = self._create_test_model(
            base_class=self.TestModelChild, model_name='TestModelProxy',
            options={
                'proxy': True
            }
        )

        ModelPermission.register(
            model=self.TestModelParent, permissions=(
                self.test_permission,
            )
        )
        ModelPermission.register_inheritance(
            model=self.TestModelChild, related='parent',
        )

        parent = self.TestModelParent.objects.create()
        child = self.TestModelChild.objects.create(parent=parent)

        self.grant_access(
            obj=parent, permission=self.test_permission
        )

        proxy_object = self.TestModelProxy.objects.get(pk=child.pk)

        self.assertTrue(
            proxy_object in AccessControlList.objects.restrict_queryset(
                permission=self.test_permission,
                queryset=self.TestModelProxy.objects.all(),
                user=self._test_case_user
            )
        )



from __future__ import unicode_literals

from mayan.apps.common.tests.base import BaseTestCase

from ..compressed_files import Archive, TarArchive, ZipArchive

from .literals import (
    TEST_ARCHIVE_ZIP_CP437_MEMBER_PATH,
    TEST_ARCHIVE_ZIP_SPECIAL_CHARACTERS_FILENAME_MEMBER_PATH,
    TEST_COMPRESSED_FILE_CONTENTS, TEST_FILE_CONTENTS_1, TEST_FILE3_PATH,
    TEST_FILENAME1, TEST_FILENAME3, TEST_TAR_BZ2_FILE_PATH,
    TEST_TAR_FILE_PATH, TEST_TAR_GZ_FILE_PATH, TEST_ZIP_FILE_PATH
)


class ArchiveClassTestCaseMixin(object):
    archive_path = None
    cls = None
    filename = TEST_FILENAME3
    file_path = TEST_FILE3_PATH
    members_list = TEST_COMPRESSED_FILE_CONTENTS
    member_name = TEST_FILENAME1
    member_contents = TEST_FILE_CONTENTS_1

    def test_add_file(self):
        archive = self.cls()
        archive.create()
        with open(self.file_path, mode='rb') as file_object:
            archive.add_file(file_object=file_object, filename=self.filename)
            self.assertTrue(archive.members(), [self.filename])

    def test_open(self):
        with open(self.archive_path, mode='rb') as file_object:
            archive = Archive.open(file_object=file_object)
            self.assertTrue(isinstance(archive, self.cls))

    def test_members(self):
        with open(self.archive_path, mode='rb') as file_object:
            archive = Archive.open(file_object=file_object)
            self.assertEqual(archive.members(), self.members_list)

    def test_member_contents(self):
        with open(self.archive_path, mode='rb') as file_object:
            archive = Archive.open(file_object=file_object)
            self.assertEqual(
                archive.member_contents(filename=self.member_name),
                self.member_contents
            )

    def test_open_member(self):
        with open(self.archive_path, mode='rb') as file_object:
            archive = Archive.open(file_object=file_object)
            file_object = archive.open_member(filename=self.member_name)
            self.assertEqual(
                file_object.read(), self.member_contents
            )


class TarArchiveClassTestCase(ArchiveClassTestCaseMixin, BaseTestCase):
    archive_path = TEST_TAR_FILE_PATH
    cls = TarArchive


class ZipArchiveClassTestCase(ArchiveClassTestCaseMixin, BaseTestCase):
    archive_path = TEST_ZIP_FILE_PATH
    cls = ZipArchive

    def test_open_member_with_special_characters_filename(self):
        with open(TEST_ARCHIVE_ZIP_SPECIAL_CHARACTERS_FILENAME_MEMBER_PATH, mode='rb') as file_object:
            archive = Archive.open(file_object=file_object)
            list(archive.get_members())

    def test_open_cp437_member(self):
        with open(TEST_ARCHIVE_ZIP_CP437_MEMBER_PATH, mode='rb') as file_object:
            archive = Archive.open(file_object=file_object)
            list(archive.get_members())


class TarGzArchiveClassTestCase(ArchiveClassTestCaseMixin, BaseTestCase):
    archive_path = TEST_TAR_GZ_FILE_PATH
    cls = TarArchive


class TarBz2ArchiveClassTestCase(ArchiveClassTestCaseMixin, BaseTestCase):
    archive_path = TEST_TAR_BZ2_FILE_PATH
    cls = TarArchive



from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0001_initial'),
        ('permissions', '0002_auto_20150628_0533'),
        ('acls', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='AccessControlList',
            fields=[
                (
                    'id', models.AutoField(
                        verbose_name='ID', serialize=False, auto_created=True,
                        primary_key=True
                    )
                ),
                ('object_id', models.PositiveIntegerField()),
                (
                    'content_type', models.ForeignKey(
                        related_name='object_content_type',
                        to='contenttypes.ContentType'
                    )
                ),
                (
                    'permissions', models.ManyToManyField(
                        related_name='acls', verbose_name='Permissions',
                        to='permissions.StoredPermission', blank=True
                    )
                ),
                (
                    'role', models.ForeignKey(
                        related_name='acls', verbose_name='Role',
                        to='permissions.Role'
                    )
                ),
            ],
            options={
                'verbose_name': 'Access entry',
                'verbose_name_plural': 'Access entries',
            },
            bases=(models.Model,),
        ),
        migrations.RemoveField(
            model_name='accessentry',
            name='content_type',
        ),
        migrations.RemoveField(
            model_name='accessentry',
            name='holder_type',
        ),
        migrations.RemoveField(
            model_name='accessentry',
            name='permission',
        ),
        migrations.DeleteModel(
            name='AccessEntry',
        ),
        migrations.DeleteModel(
            name='CreatorSingleton',
        ),
        migrations.RemoveField(
            model_name='defaultaccessentry',
            name='content_type',
        ),
        migrations.RemoveField(
            model_name='defaultaccessentry',
            name='holder_type',
        ),
        migrations.RemoveField(
            model_name='defaultaccessentry',
            name='permission',
        ),
        migrations.DeleteModel(
            name='DefaultAccessEntry',
        ),
        migrations.AlterUniqueTogether(
            name='accesscontrollist',
            unique_together=set([('content_type', 'object_id', 'role')]),
        ),
    ]



from __future__ import unicode_literals

from django.utils.module_loading import import_string

from .settings import setting_ocr_backend, setting_ocr_backend_arguments

ocr_backend = import_string(
    dotted_path=setting_ocr_backend.value
)(**setting_ocr_backend_arguments.value)



from __future__ import unicode_literals

import logging

from django.urls import reverse
from django.utils.translation import ugettext_lazy as _
from django.views.generic.base import RedirectView

from mayan.apps.common.generics import SimpleView, SingleObjectListView
from mayan.apps.common.literals import LIST_MODE_CHOICE_ITEM

from .forms import SearchForm, AdvancedSearchForm
from .icons import icon_search_submit
from .mixins import SearchModelMixin

logger = logging.getLogger(__name__)


class ResultsView(SearchModelMixin, SingleObjectListView):
    def get_extra_context(self):
        context = {
            'hide_object': True,
            'no_results_icon': icon_search_submit,
            'no_results_text': _(
                'Try again using different terms. '
            ),
            'no_results_title': _('No search results'),
            'search_model': self.search_model,
            'title': _('Search results for: %s') % self.search_model.label,
        }

        if self.search_model.list_mode == LIST_MODE_CHOICE_ITEM:
            context['list_as_items'] = True

        return context

    def get_source_queryset(self):
        self.search_model = self.get_search_model()

        if self.request.GET:
            # Only do search if there is user input, otherwise just render
            # the template with the extra_context

            if self.request.GET.get('_match_all', 'off') == 'on':
                global_and_search = True
            else:
                global_and_search = False

            queryset = self.search_model.search(
                global_and_search=global_and_search,
                query_string=self.request.GET, user=self.request.user
            )

            return queryset


class SearchView(SearchModelMixin, SimpleView):
    template_name = 'appearance/generic_form.html'
    title = _('Search')

    def get_extra_context(self):
        self.search_model = self.get_search_model()
        return {
            'form': self.get_form(),
            'form_action': reverse(
                viewname='search:results', kwargs={
                    'search_model': self.search_model.get_full_name()
                }
            ),
            'search_model': self.search_model,
            'submit_icon_class': icon_search_submit,
            'submit_label': _('Search'),
            'submit_method': 'GET',
            'title': _('Search for: %s') % self.search_model.label,
        }

    def get_form(self):
        if ('q' in self.request.GET) and self.request.GET['q'].strip():
            query_string = self.request.GET['q']
            return SearchForm(initial={'q': query_string})
        else:
            return SearchForm()


class AdvancedSearchView(SearchView):
    title = _('Advanced search')

    def get_form(self):
        return AdvancedSearchForm(
            data=self.request.GET, search_model=self.get_search_model()
        )


class SearchAgainView(RedirectView):
    pattern_name = 'search:search_advanced'
    query_string = True
